# Formatting configuration for locale eo

languages={'gu': u'gu\u011darata', 'gd': 'gaela', 'ga': 'irlanda', 'gn': 'gvarania', 'gl': 'galega', 'la': 'latino', 'ln': 'lingala', 'lo': u'la\u016da', 'tt': 'tatara', 'tr': 'turka', 'ts': 'conga', 'lv': 'latva', 'to': 'tongaa', 'lt': 'litova', 'tk': 'turkmena', 'th': 'taja', 'ti': 'tigraja', 'tg': u'ta\u011dika', 'te': 'telugua', 'ta': 'tamila', 'yi': 'jida', 'yo': 'joruba', 'de': 'germana', 'da': 'dana', 'dz': 'dzonko', 'st': 'sota', 'qu': u'ke\u0109ua', 'el': 'greka', 'eo': 'esperanto', 'en': 'angla', 'zh': u'\u0109ina', 'za': u'\u011duanga', 'uk': 'ukraina', 'eu': u'e\u016dska', 'et': 'estona', 'es': 'hispana', 'ru': 'rusa', 'rw': 'ruanda', 'rm': u'roman\u0109a', 'rn': 'burunda', 'ro': 'rumana', 'bn': 'bengala', 'be': 'belorusa', 'bg': 'bulgara', 'ba': u'ba\u015dkira', 'wo': 'volofa', 'jv': 'java', 'bo': 'tibeta', 'bh': 'bihara', 'bi': 'bislamo', 'br': 'bretona', 'ja': 'japana', 'om': 'oroma', 'oc': 'okcitana', 'tw': 'akana', 'or': 'orijo', 'xh': 'ksosa', 'co': 'korsika', 'ca': 'kataluna', 'cy': 'kimra', 'cs': u'\u0109e\u0125a', 'ps': u'pa\u015dtua', 'pt': 'portugala', 'tl': 'filipina', 'pa': u'pan\u011daba', 'vi': 'vjetnama', 'pl': 'pola', 'hy': 'armena', 'hr': 'kroata', 'iu': 'inuita', 'hu': 'hungara', 'hi': 'hinda', 'ha': u'ha\u016dsa', 'he': 'hebrea', 'mg': 'malagasa', 'uz': 'uzbeka', 'ml': 'malajalama', 'mn': 'mongola', 'mi': 'maoria', 'ik': 'eskima', 'mk': 'makedona', 'ur': 'urduo', 'mt': 'malta', 'ms': 'malaja', 'mr': 'marata', 'ug': 'ujgura', 'my': 'birma', 'aa': 'afara', 'ab': u'ab\u0125aza', 'ss': 'svazia', 'af': 'afrikansa', 'tn': 'cvana', 'sw': 'svahila', 'is': 'islanda', 'am': 'amhara', 'it': 'itala', 'sv': 'sveda', 'ia': 'interlingvao', 'as': 'asama', 'ar': 'araba', 'su': 'sunda', 'zu': 'zulua', 'ay': 'ajmara', 'az': u'azerbaj\u011dana', 'ie': 'okcidentalo', 'id': 'indonezia', 'sk': 'slovaka', 'nl': 'nederlanda', 'no': 'norvega', 'na': 'naura', 'ne': 'nepala', 'vo': 'volapuko', 'fr': 'franca', 'sm': 'samoa', 'fy': 'frisa', 'fa': 'persa', 'fi': 'finna', 'fj': u'fi\u011dia', 'sa': 'sanskrito', 'fo': 'feroa', 'ka': 'kartvela', 'kk': u'kaza\u0125a', 'sr': 'serba', 'sq': 'albana', 'ko': 'korea', 'kn': 'kanara', 'km': 'kmera', 'kl': 'gronlanda', 'ks': u'ka\u015dmira', 'si': 'sinhala', 'sh': 'serbo-Kroata', 'so': 'somala', 'sn': u'\u015dona', 'ku': 'kurda', 'sl': 'slovena', 'ky': 'kirgiza', 'sg': 'sangoa', 'sd': 'sinda'}

countries={'BD': u'Banglade\u015do', 'BE': 'Belgujo', 'BF': 'Burkino', 'BG': 'Bulgarujo', 'BA': 'Bosnio-Hercegovino', 'BB': 'Barbado', 'WF': 'Valiso kaj Futuno', 'BM': 'Bermudoj', 'BN': 'Brunejo', 'BO': 'Bolivio', 'BH': 'Barejno', 'BI': 'Burundo', 'BJ': 'Benino', 'BT': 'Butano', 'JM': 'Jamajko', 'BW': 'Bocvano', 'WS': 'Samoo', 'BR': 'Brazilo', 'BS': 'Bahamoj', 'BY': 'Belorusujo', 'BZ': 'Belizo', 'RU': 'Rusujo', 'RW': 'Ruando', 'RE': 'Reunio', 'TM': 'Turkmenujo', 'TJ': u'Ta\u011dikujo', 'RO': 'Rumanujo', 'GW': u'Gvineo-Bisa\u016do', 'GU': 'Gvamo', 'GT': 'Gvatemalo', 'GS': u'Sud-Georgio kaj Sud-Sandvi\u0109insuloj', 'GR': 'Grekujo', 'GQ': 'Ekvatora Gvineo', 'GP': 'Gvadelupo', 'JP': 'Japanujo', 'GY': 'Gujano', 'GF': 'Franca Gviano', 'GE': 'Kartvelujo', 'GD': 'Grenado', 'GB': u'Unui\u011dinta Re\u011dlando', 'GA': 'Gabono', 'SV': 'Salvadoro', 'GN': 'Gvineo', 'GM': 'Gambio', 'GL': 'Gronlando', 'GI': u'\u011cibraltaro', 'GH': 'Ganao', 'OM': 'Omano', 'TN': 'Tunizio', 'JO': 'Jordanio', 'SP': 'Serbujo', 'HR': 'Kroatujo', 'HT': 'Haitio', 'HU': 'Hungarujo', 'HN': 'Honduro', 'HM': 'Herda kaj Makdonaldaj Insuloj', 'VE': 'Venezuelo', 'PR': 'Puerto-Riko', 'PW': u'Bela\u016do', 'PT': 'Portugalujo', 'SJ': 'Svalbardo kaj Jan-Majen-insulo', 'PY': 'Paragvajo', 'IQ': 'Irako', 'PA': 'Panamo', 'PF': 'Franca Polinezio', 'PG': 'Papuo-Nov-Gvineo', 'PE': 'Peruo', 'PK': 'Pakistano', 'PH': 'Filipinoj', 'PN': 'Pitkarna Insulo', 'PL': 'Pollando', 'PM': 'Sent-Piero kaj Mikelono', 'ZM': 'Zambio', 'EH': 'Okcidenta Saharo', 'EE': 'Estonujo', 'EG': 'Egipto', 'ZA': 'Sud-Afriko', 'EC': 'Ekvadoro', 'IT': 'Italujo', 'VN': 'Vjetnamo', 'SB': 'Salomonoj', 'ET': 'Etiopujo', 'SO': 'Somalujo', 'ZW': 'Zimbabvo', 'SA': u'Sa\u016dda Arabujo', 'ES': 'Hispanujo', 'ER': 'Eritreo', 'MD': 'Moldavujo', 'MG': 'Madagaskaro', 'MA': 'Maroko', 'MC': 'Monako', 'UZ': 'Uzbekujo', 'MM': 'Mjanmao', 'ML': 'Malio', 'MN': 'Mongolujo', 'MH': u'Mar\u015daloj', 'MK': 'Makedonujo', 'MU': u'Ma\u016dricio', 'MT': 'Malto', 'MW': 'Malavio', 'MV': 'Maldivoj', 'MQ': 'Martiniko', 'MP': 'Nord-Marianoj', 'MR': u'Ma\u016dritanujo', 'UG': 'Ugando', 'MY': 'Malajzio', 'MX': 'Meksiko', 'IL': 'Israelo', 'FR': 'Francujo', 'IO': 'Brita Hindoceana Teritorio', 'SH': 'Sent-Heleno', 'FI': 'Finnlando', 'FJ': u'Fi\u011doj', 'FM': 'Mikronezio', 'FO': 'Ferooj', 'NI': 'Nikaragvo', 'NL': 'Nederlando', 'NO': 'Norvegujo', 'NA': 'Namibio', 'VU': 'Vanuatuo', 'NC': 'Nov-Kaledonio', 'NE': u'Ni\u011dero', 'NF': 'Norfolkinsulo', 'NG': u'Ni\u011derio', 'NZ': 'Nov-Zelando', 'NP': 'Nepalo', 'NR': 'Nauro', 'NU': 'Niuo', 'CK': 'Kukinsuloj', 'CI': 'Ebur-Bordo', 'CH': 'Svisujo', 'CO': 'Kolombio', 'CN': u'\u0108inujo', 'CM': 'Kameruno', 'CL': u'\u0108ilio', 'CA': 'Kanado', 'CG': 'Kongolo', 'CF': 'Centr-Afrika Respubliko', 'CZ': u'\u0108e\u0125ujo', 'CY': 'Kipro', 'CR': 'Kostariko', 'CV': 'Kabo-Verdo', 'CU': 'Kubo', 'SZ': 'Svazilando', 'SY': 'Sirio', 'KG': 'Kirgizistano', 'KE': 'Kenjo', 'SR': 'Surinamo', 'KI': 'Kiribato', 'KH': u'Kambo\u011do', 'KN': 'Sent-Kristofo kaj Neviso', 'KM': 'Komoroj', 'ST': 'Sao-Tomeo kaj Principeo', 'SK': 'Slovakujo', 'KR': 'Sud-Koreo', 'SI': 'Slovenujo', 'KP': 'Nord-Koreo', 'KW': 'Kuvajto', 'SN': 'Senegalo', 'SM': 'San-Marino', 'SL': 'Siera-Leono', 'SC': u'Sej\u015deloj', 'KZ': u'Kaza\u0125stano', 'KY': 'Kejmanoj', 'SG': 'Singapuro', 'SE': 'Svedujo', 'SD': 'Sudano', 'DO': 'Domingo', 'DM': 'Dominiko', 'DJ': u'\u011cibutio', 'DK': 'Danujo', 'VG': 'Britaj Virgulininsuloj', 'DE': 'Germanujo', 'YE': 'Jemeno', 'DZ': u'Al\u011derio', 'US': 'Usono', 'UY': 'Urugvajo', 'YT': 'Majoto', 'UM': 'Usonaj malgrandaj insuloj', 'LB': 'Libano', 'LC': 'Sent-Lucio', 'LA': 'Laoso', 'TV': 'Tuvalo', 'TW': 'Tajvano', 'TT': 'Trinidado kaj Tobago', 'TR': 'Turkujo', 'LK': 'Sri-Lanko', 'LI': u'Li\u0125ten\u015dtejno', 'LV': 'Latvujo', 'TO': 'Tongo', 'LT': 'Litovujo', 'LU': 'Luksemburgo', 'LR': 'Liberio', 'LS': 'Lesoto', 'TH': 'Tajlando', 'TG': 'Togolo', 'TD': u'\u0108ado', 'LY': 'Libio', 'VA': 'Vatikano', 'VC': 'Sent-Vincento kaj la Grenadinoj', 'AE': u'Unui\u011dintaj Arabaj Emirlandos', 'AD': 'Andoro', 'AG': 'Antigvo-Barbudo', 'AF': 'Afganujo', 'AI': 'Angvilo', 'VI': 'Usonaj Virgulininsuloj', 'IS': 'Islando', 'IR': 'Irano', 'AM': 'Armenujo', 'AL': 'Albanujo', 'AO': 'Angolo', 'AN': 'Nederlandaj Antiloj', 'AQ': 'Antarkto', 'AR': 'Argentino', 'AU': u'A\u016dstralio', 'AT': u'A\u016dstrujo', 'AW': 'Arubo', 'IN': 'Hindujo', 'TZ': 'Tanzanio', 'AZ': u'Azerbaj\u011dano', 'IE': 'Irlando', 'ID': 'Indonezio', 'UA': 'Ukrajno', 'QA': 'Kataro', 'MZ': 'Mozambiko'}

months=['januaro', 'februaro', 'marto', 'aprilo', 'majo', 'junio', 'julio', u'a\u016dgusto', 'septembro', 'oktobro', 'novembro', 'decembro']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', u'a\u016dg', 'sep', 'okt', 'nov', 'dec']

days=['lundo', 'mardo', 'merkredo', u'\u0135a\u016ddo', 'vendredo', 'sabato', u'diman\u0109o']

abbrDays=['lu', 'ma', 'me', u'\u0135a', 've', 'sa', 'di']

dateFormats={'medium': '%Y-%%(abbrmonthname)s-%d', 'full': '%%(dayname)s, %d-a de %%(monthname)s %Y', 'long': '%Y-%%(monthname)s-%d', 'short': '%y-%m-%d'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


from gettext import translation
import turbogears
from turbogears.i18n.utils import get_locale
from turbojson.jsonify import jsonify
from turbogears.util import request_available
import os
import types

_catalogs = {}

def is_locale_supported(locale, domain=None):
    """Checks if [domain].mo file exists for this language
    """

    if not domain:domain = turbogears.config.get("i18n.domain", "messages")
    localedir = turbogears.config.get("i18n.locale_dir", "locales")

    return os.path.exists(os.path.join(localedir, locale, "LC_MESSAGES", "%s.mo" %domain))

def get_catalog(locale, domain = None):
    """Return translations for given locale. 
    """
    if domain is None:domain = turbogears.config.get("i18n.domain", "messages")

    catalog = _catalogs.get(domain)

    if not catalog:

        catalog = _catalogs[domain] = {}

    messages = catalog.get(locale)
    if not messages:

        localedir = turbogears.config.get("i18n.locale_dir", "locales")
        messages = catalog[locale] = translation(domain=domain, 
            localedir=localedir, languages=[locale])

    return messages

def plain_gettext(key, locale=None, domain=None):
    """Gets the gettext value for key.Added to builtins as '_'. Returns Unicode string.
    @param key: text to be translated
    @param locale: locale code to be used.If locale is None, gets the value provided 
    by get_locale.
    """
    gettext_func = turbogears.config.get("i18n.gettext", tg_gettext)
    return gettext_func(key, locale, domain)

def tg_gettext(key, locale=None, domain=None):
    """Gets the gettext value for key.Added to builtins as '_'. Returns Unicode string.
    @param key: text to be translated
    @param locale: locale code to be used.If locale is None, gets the value provided by 
    get_locale.
    """
    if locale is None:locale = get_locale()
    if not is_locale_supported(locale):locale = locale[:2]
    if key == '': return '' # special case

    try:
        return get_catalog(locale, domain).ugettext(key)
    except KeyError:
        return key
    except IOError:
        return key

def plain_ngettext(key1, key2, num, locale=None):
    """Translates two possible texts based on whether num is greater than 1
    @param key1: text if num==1
    @param key2: text if num!=1
    @param num: a number
    @type num: integer
    @locale:locale code to be used.If locale is None, gets the value provided by get_locale.
    """

    if num==1:

        return plain_gettext(key1, locale) 

    else:

        return plain_gettext(key2, locale) 

class lazystring(object):
    """Has a number of lazily evaluated functions replicating a string. Just override the eval() method to produce the actual value.
    """

    def __init__(self, func, *args, **kw):

        self.func = func
        self.args = args
        self.kw = kw

    def eval(self):

        return self.func(*self.args, **self.kw)

    def __unicode__(self):
        return unicode(self.eval())

    def __str__(self):
        return str(self.eval())

    def __mod__(self, other):
        return self.eval() % other

    def __cmp__(self, other):
        return cmp(self.eval(), other)

    def __eq__(self, other):
        return self.eval() == other

    def __deepcopy__(self, memo):
        return self

def lazify(func):

    def newfunc(*args, **kw):

        lazystr = lazystring(func, *args, **kw)
        return lazystr

    return newfunc

def jsonify_lazystring(obj):
    return unicode(obj)
jsonify.when("isinstance(obj, lazystring)")(jsonify_lazystring)

lazy_gettext = lazify(plain_gettext)
lazy_ngettext = lazify(plain_ngettext)

def gettext(key, locale=None, domain=None):
    """Gets the gettext value for key. Added to builtins as '_'. Returns Unicode string.

    @param key: text to be translated
    @param locale: locale code to be used. If locale is None, gets the value provided by get_locale.
     """

    if request_available():
        return plain_gettext(key, locale, domain)
    else:
        return lazy_gettext(key, locale, domain)

def ngettext(key1, key2, num, locale=None):
    """Translates two possible texts based on whether num is greater than 1.

    @param key1: text if num==1
    @param key2: text if num!=1
    @param num: a number
    @type num: integer
    @param locale: locale code to be used. If locale is None, gets the value provided by get_locale.
    """
    if request_available():
        return plain_ngettext(key1, key2, num, locale)
    else:
        return lazy_ngettext(key1, key2, num, locale)

def install():
    """Adds the gettext function to __builtins__ as '_'
    """
    __builtins__["_"] = gettext



"""Graphical user interface for SQLObject's model design"""
import os
import string
import shutil

import turbogears
from turbogears import controllers
import cherrypy
import sqlobject
import pkg_resources

model_template = """from datetime import datetime
from turbogears.database import PackageHub
from sqlobject import *
from turbogears import identity 

hub = PackageHub("%s")
__connection__ = hub

%s
"""

session_file_name = 'model_designer.tmp'

def version_file(file_spec,vtype='copy'):
    """By Robin Parmar & Martin Miller from Python Cookbook 2. edition"""
    
    if os.path.isfile(file_spec):
        #check the 'vtype' parameter
        if vtype not in ('copy', 'rename'):
            raise ValueError, 'Unknown vtype %r' % (vtype,)

        # determine root file name so the extension doesn't get longer and longer...
        n, e = os.path.splitext(file_spec)

        # is e a three-digets integer preceded by a dot?
        if len(e) == 4 and e[1:].isdigit():
            num = 1 + int(e[1:])
            root = n
        else:
            num = 0
            root = file_spec

        # Find next available file version
        for i in xrange(num,1000):
            new_file = '%s.%03d' % (root, i)
            if not os.path.exists(new_file):
                if vtype == 'copy':
                    shutil.copy(file_spec, new_file)
                else:
                    os.rename(file_spec, new_file)
                return True
        raise RuntimeError, "Can't %s %r, all names taken"% (vtype,file_spec)
    return False

class Designer(controllers.RootController):
    """Designer for SQLObject models. 
       Create your Classes, define your fields and manage your relations.
       Visualize and generate code for SQLObject models.  
    """
    __label__ ="ModelDesigner"
    __version__ = "0.1"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"

    baseTemplate = 'turbogears.toolbox.designer'
    model = None
    icon = "/tg_static/images/designer.png"

    def set_model(self,model):
        self.model = model

    def get_model_name(self):
        if not turbogears.util.get_project_name(): return False
        return turbogears.util.get_model()
        
    def register_static_directory(self):
        static_directory = pkg_resources.resource_filename(__name__,'static')
        turbogears.config.update( 
                                { '/tg_toolbox/designer': {
                                                            'static_filter.on':True,
                                                            'static_filter.dir':static_directory
                                                          }
                                }
                               )


    def column_type(self,column):
        """Given a column representation return the column type"""
        column_type = '%r'% column
        return column_type.split()[0][1:].split('.')[-1]

    def column_default(self,column,column_type):
        try: default = column.default
        except: return ''
        if default== sqlobject.sqlbuilder.NoDefault:return ''
        if column_type in ('SOIntCol',
                           'SOFloatCol',
                           'SOStringCol',
                           'SODecimalCol',
                           'SOCurrencyCol'):return default
        if column_type == 'SODateTimeCol':
            d = '%s'% default
            return ':'.join( d.split(':')[:-1] )
        if column_type == 'SODateCol':
            d = '%s'% default
            return d.split()[0] 
        return ''

    def load_column(self,column_name,column,model_object):
        props= {} 
        
        props['type'] = self.column_type(column)
        props['column_name'] = column_name
        props['name'] = column_name
        props['column_default'] = self.column_default(column,props['type'])
        try: 
            props['title'] =( getattr(column,'title') or '')
        except: 
            props['title'] = ''
        if props['type'] == 'SOStringCol': 
            props['length'] = column.length
            props['varchar'] = column.varchar
        if props['type'] == 'SOEnumCol': props['enum_values'] = column.enumValues
        if props['type'] == 'SOForeignKey':
            props['join_type'] = 'MultipleJoin'
            props['other_class_name'] = column.foreignKey
            props['other_method_name'] = self.other_method_name(column,model_object.__name__) 
        if props['type'] in ('SOMultipleJoin','SORelatedJoin'):
            props['other_class_name'] = column.otherClassName
            props['join_type'] = props['type'].replace('SO','')
            props['other_method_name'] = self.other_method_join_name(column,model_object.__name__) 

        props['type'] = props['type'].replace('SO','')
        return props

    def other_method_join_name(self,column,model_object_name):
        for col in column.otherClass.sqlmeta.columnList:
            if type(col) == sqlobject.SOForeignKey:
                print 'col:',col
                print 'colfk:',col.foreignKey
                print 'moname:',model_object_name
                if col.foreignKey == model_object_name:
                    return col.origName
        return 'id'

    def other_method_name(self,column,model_object_name):
        other_class_name = column.foreignKey
        model_object = getattr(self.get_model_name(),other_class_name)
        for col in model_object.sqlmeta.joins:
            if col.otherClassName == model_object_name:
                return col.joinMethodName
        return 'id'

    def is_inheritable_base_class(self,obj):
        '''Check if the object is  a direct subclass of InheritableSQLObject'''
        return 'sqlobject.inheritance.InheritableSQLObject' in '%s'% obj.__bases__

    def load_columns(self,model_object):
        columns = {}
        """get normal columns"""
        for column_name in model_object.sqlmeta.columns:
            column = model_object.sqlmeta.columns[column_name]
            origname = column.origName
            if model_object._inheritable and column_name == 'childName':continue
            columns[origname] = self.load_column(origname,column,model_object) 

        """get join columns"""
        for column in model_object.sqlmeta.joins:
            columns[column.joinMethodName] = self.load_column(column.joinMethodName,column,model_object) 

        return columns

    def load_current_model(self):
        model = self.get_model_name() 
        current_model = {}
        current_model['name'] = turbogears.util.get_package_name() 
        current_model['models'] = {}
        current_model['ordered_models'] =[]

        for m in dir(model):
            if m in ('SQLObject','InheritableSQLObject'): continue
            model_object = getattr(model,m)
            if isinstance(model_object, type) and issubclass(model_object,sqlobject.SQLObject):
                parent_class = 'SQLObject'
                if model_object._inheritable: parent_class = model_object._parentClass.__name__
                columns = self.load_columns(model_object)

                current_model['ordered_models'].append(m)
                current_model['models'][m] = {
                                                'name':m,
                                                'parent_class':parent_class,
                                                'table_name':model_object.sqlmeta.table,
                                                'id_name':model_object.sqlmeta.idName,
                                                'columns':columns,
                                                'relations':{}
                                             }
        return dict(model=current_model)
    load_current_model= turbogears.expose(format='json')(load_current_model)

    def save_session_as_name(self,name):
        name = name.translate(string.maketrans('',''), string.punctuation) 
        name =  ''.join([x.title() for x in name.split()]) #camelcase to remove spaces
        sessions_directory = pkg_resources.resource_filename(__name__,os.path.join('static','sessions'))
        postfix = ''
        for idx in range(100):
            if idx > 0: postfix = '_%s'% idx
            test_name = '%s%s.js'% (name,postfix) 
            full_name = os.path.join(sessions_directory,test_name)
            if not os.path.exists(full_name): return full_name

    def save_state(self,state,name=session_file_name):
        if name != session_file_name: name = self.save_session_as_name(name)
        if not name:return dict(state=False)
            
        f = open(name,'w')
        f.write(state)
        f.close()
        return dict(state=True)
    save_state = turbogears.expose(format='json')(save_state)

    def model_path(self):
        return os.path.abspath( os.path.join(turbogears.util.get_package_name(),'model.py') )

    def save_model(self,code):
        project_name = turbogears.util.get_package_name() 
        model_text = model_template % (project_name,code) 
        model_path = self.model_path() 
        if os.path.exists(model_path):
            version_file(model_path)
            f=open(model_path,'w')
            f.write(model_text)
            f.close()
            return True,'Saved %s'% model_path
        else:
            return False,'Failed to save %s'% model_path 

    def save_and_create_(self,code,order):
        project_name = turbogears.util.get_package_name() 
        model_path = os.path.join(turbogears.util.get_project_name(),'tmp2_model.py')
        model_path = os.path.abspath(model_path)
        open(model_path,'w').write(str)

        package = __import__(turbogears.util.get_package_name(), {}, {}, ["tmp2_model"])
        print package.tmp2_model.Persona
        package.tmp2_model.Persona.createTable(ifNotExists=True)
        model = self.get_model_name()
        model.Persona = package.tmp2_model.Persona
        #pk = package.model
        #import tmp_model as pk
        #pk.Persona.createTable(ifNotExists=True)
        return dict(status='Tables ok')

    def save_and_create(self,code,order):
        status_code,status = self.save_model(code)
        if not status_code: return dict(status=status)
        tmp_name = 'tmp_model'

        #copy the model file
        model_path = self.model_path() 
        tmp_model_path = model_path.replace('model.py','%s.py'% tmp_name)
        try:
            shutil.copy(model_path, tmp_model_path)
        except IOError, e:
            return dict(status='Failed to create a temporary model file:%s'% e)

        model = self.get_model_name()
        package = __import__(turbogears.util.get_package_name(), {}, {}, [tmp_name])
        tmp_model = getattr(package,tmp_name)
        print tmp_model
            
        ok = []
        fail = []
        classes = order.split(',')
        for class_name in classes:
            try:
                obj = getattr(tmp_model,class_name)
                obj.createTable(ifNotExists=True)
                setattr(model,class_name,obj)
                ok.append('Table created for class %s'% class_name)
            except Exception, e:
                fail.append('Failed to create table for class %s:%s'% (class_name,e))
        
        if len(ok): status += '\nTables Created: \n%s'% '\n'.join(ok) 
        if len(fail): status+= '\nFailed to create Tables:\n%s'% '\n'.join(fail)

        try:
            os.remove(tmp_model_path)
        except IOError, e:
            print "Fail to remove temporary model file:%s"% e 

        return dict(status=status)
    save_and_create = turbogears.expose(format='json')(save_and_create)
        
    def load_session_list(self):
        sessions_directory = pkg_resources.resource_filename(__name__,os.path.join('static','sessions'))
        return [x.replace('.js','') for x in os.listdir(sessions_directory) if x.endswith('.js')]

    def session_exists(self):
        return os.path.exists(session_file_name)

    def retrieve_sample(self,name):
        sessions_directory = pkg_resources.resource_filename(__name__,os.path.join('static','sessions'))
        full_path = os.path.join(sessions_directory,'%s.js'% name)
        if not os.path.exists(full_path): return dict()
        return dict(session=open(full_path,'r').read())
    retrieve_sample = turbogears.expose(format='json')(retrieve_sample)

    def current_session_model(self):
        if not self.session_exists(): return dict() 
        return dict(session=open(session_file_name,'r').read())
    current_session_model = turbogears.expose(format='json')(current_session_model)

    def save(self,code):
        status_code,status = self.save_model(code)
        return dict(status=status)
    save= turbogears.expose(format='json')(save)

    def session_list(self):
        return dict(models=self.load_session_list())
    session_list = turbogears.expose(format='json')(session_list)

    def index(self):
        print cherrypy.request.path
        self.register_static_directory()
        self.model = self.get_model_name() 
        model_exists = None 
        if self.model: model_exists = 1
        return dict(model_exists=model_exists,
                    session_file_exists = self.session_exists(),
                    session_list = self.load_session_list(),
                    model_name=turbogears.util.get_project_name())
    index = turbogears.expose(template='%s.modelDesigner'% baseTemplate)(index)



"Things to do when TurboGears is imported."

import os
import errno
import logging
import sys
import time
import atexit
import signal

import pkg_resources
import cherrypy
from cherrypy import _cputil
from formencode.variabledecode import NestedVariables
from cherrypy._cpwsgi import wsgiApp, CPHTTPRequest
from cherrypy._cpwsgiserver import CherryPyWSGIServer

from turbogears import config, scheduler, database
from turbogears import view
from turbogears.database import hub_registry, EndTransactionsFilter

log = logging.getLogger("turbogears.startup")

pkg_resources.require("TurboGears")


def reloader_thread(freq):
    """Monkeypatch for the reloader provided by CherryPy.

    This reloader is designed to reload a single package. This is
    more efficient and, more important, compatible with zipped
    libraries that may not provide access to the individual files."""

    def archive_selector(module):
        if hasattr(module, '__loader__'):
            if hasattr(module.__loader__, 'archive'):
                return module.__loader__.archive
        return module

    mtimes = {}
    package = config.get("autoreload.package", None)
    if package is None:
        print \
"""TurboGears requires autoreload.package to be set. It can be an empty
value, which will use CherryPy's default behavior which is to check
every module. Setting an actual package makes the check much faster."""
        return
    while cherrypy.lib.autoreload.RUN_RELOADER:
        if package:
            modnames = filter(lambda modname: modname.startswith(package),
                                sys.modules.keys())
            modlist = [sys.modules[modname] for modname in modnames]
        else:
            modlist = map(archive_selector, sys.modules.values())
        for filename in filter(lambda v: v,
                map(lambda m: getattr(m, "__file__", None), modlist)):
            if filename.endswith(".kid") or filename == "<string>":
                continue
            orig_filename = filename
            if filename.endswith(".pyc"):
                filename = filename[:-1]
            try:
                mtime = os.stat(filename).st_mtime
            except OSError, e:
                if orig_filename.endswith('.pyc') and e[0] == errno.ENOENT:
                    # This prevents us from endlessly restarting
                    # if there is an old .pyc lying around
                    # after a .py file has been deleted
                    try: os.unlink(orig_filename)
                    except: pass
                sys.exit(3) # force reload
            if filename not in mtimes:
                mtimes[filename] = mtime
                continue
            if mtime > mtimes[filename]:
                sys.exit(3) # force reload
        time.sleep(freq)

cherrypy.lib.autoreload.reloader_thread = reloader_thread

webpath = ""

DNS_SD_PID = None


def start_bonjour(package=None):
    global DNS_SD_PID
    if DNS_SD_PID:
        return
    if (not hasattr(cherrypy, "root")) or (not cherrypy.root):
        return
    if not package:
        package = cherrypy.root.__module__
        package = package[:package.find(".")]

    host = config.get('server.socket_host', '')
    port = str(config.get('server.socket_port'))
    env = config.get('server.environment')
    name = package + ": " + env
    type = "_http._tcp"

    cmds = [['/usr/bin/avahi-publish-service', ["-H", host, name, type, port]],
            ['/usr/bin/dns-sd', ['-R', name, type, "."+host, port, "path=/"]]]

    for cmd, args in cmds:
        # TODO:. This check is flawed.  If one has both services installed and
        # avahi isn't the one running, then this won't work.  We should either
        # try registering with both or checking what service is running and use
        # that.  Program availability on the filesystem was never enough...
        if os.path.exists(cmd):
            DNS_SD_PID = os.spawnv(os.P_NOWAIT, cmd, [cmd]+args)
            atexit.register(stop_bonjour)
            break


def stop_bonjour():
    if not DNS_SD_PID:
        return
    try:
        os.kill(DNS_SD_PID, signal.SIGTERM)
    except OSError:
        pass


class VirtualPathFilter(object):
    """Filter that makes CherryPy ignorant of a URL root path.

    That is, you can mount your app so the URI "/users/~rdel/myapp/"
    maps to the root object "/".
    """

    def on_start_resource(self):
        prefix = config.get('server.webpath', False)
        if prefix:
            path = cherrypy.request.object_path
            if path == prefix:
                cherrypy.request.object_path = '/'
            elif path.startswith(prefix):
                cherrypy.request.object_path = path[len(prefix):]
            else:
                raise cherrypy.NotFound(path)

class NestedVariablesFilter(object):

    def before_main(self):
        if hasattr(cherrypy.request, "params"):
            cherrypy.request.params = \
                NestedVariables.to_python(cherrypy.request.params or {})


def startTurboGears():
    """Handles TurboGears tasks when the CherryPy server starts.

    This adds the "tg_js" configuration to make MochiKit accessible.
    It also turns on stdlib logging when in development mode.
    """
    config.update({"/tg_static":
            {
            "static_filter.on": True,
            "static_filter.dir":
                os.path.abspath(pkg_resources.resource_filename(__name__, "static")),
            'log_debug_info_filter.on' : False,
            }
        })
    config.update({"/tg_js" :
            {
            "static_filter.on" : True,
            "static_filter.dir" :
                os.path.abspath(pkg_resources.resource_filename(__name__, "static/js")),
            'log_debug_info_filter.on' : False,
            }
        })
    cherrypy.config.environments['development']['log_debug_info_filter.on'] = False
    
    if config.get("decoding_filter.on", path="/") is None:
        config.update({"/": {
            "decoding_filter.on" : True,
            "decoding_filter.encoding" : config.get(
                                        "kid.encoding", "utf8")
        }})
        
    view.load_engines()
    view.loadBaseTemplates()
    global webpath
    webpath = config.get("server.webpath", "")

    if hasattr(cherrypy, "root") and cherrypy.root:
        if not hasattr(cherrypy.root, "_cp_filters"):
            cherrypy.root._cp_filters= []
        morefilters = [EndTransactionsFilter(),
                       NestedVariablesFilter()]
        if webpath:
            morefilters.insert(0, VirtualPathFilter())
        cherrypy.root._cp_filters.extend(morefilters)

    if webpath.startswith("/"):
        webpath = webpath[1:]
    if webpath and not webpath.endswith("/"):
        webpath = webpath + "/"
    isdev = config.get('server.environment') == 'development'
    if not config.get("tg.new_style_logging"):
        if config.get('server.log_to_screen'):
            setuplog = logging.getLogger()
            setuplog.setLevel(logging.DEBUG)
            fmt = logging.Formatter("%(asctime)s %(name)s "
                                    "%(levelname)s %(message)s")
            handler = logging.StreamHandler(sys.stdout)
            handler.setLevel(logging.DEBUG)
            handler.setFormatter(fmt)
            setuplog.addHandler(handler)
        
        logfile = config.get("server.log_file")
        if logfile:
            setuplog = logging.getLogger("turbogears.access")
            setuplog.propagate = 0
            fmt = logging.Formatter("%(message)s")
            handler = logging.FileHandler(logfile)
            handler.setLevel(logging.INFO)
            handler.setFormatter(fmt)
            setuplog.addHandler(handler)

    bonjoursetting = config.get("tg.bonjour", None)
    if bonjoursetting or isdev:
        start_bonjour(bonjoursetting)
    
    if config.get("sqlalchemy.dburi"):
        database.bind_meta_data()

    # Start all TurboGears extensions
    extensions= pkg_resources.iter_entry_points( "turbogears.extensions" )
    for entrypoint in extensions:
        ext= entrypoint.load()
        if hasattr(ext, "start_extension"):
            ext.start_extension()
    
    for item in call_on_startup:
        item()
        
    if config.get("tg.scheduler", False):
        scheduler._start_scheduler()
        log.info("Scheduler started")


def stopTurboGears():
    # end all transactions and clear out the hubs to
    # help ensure proper reloading in autoreload situations
    for hub in hub_registry:
        hub.end()
    hub_registry.clear()

    stop_bonjour()

    # Shut down all TurboGears extensions
    extensions= pkg_resources.iter_entry_points( "turbogears.extensions" )
    for entrypoint in extensions:
        ext= entrypoint.load()
        if hasattr(ext, "shutdown_extension"):
            ext.shutdown_extension()

    for item in call_on_shutdown:
        item()
        
    if config.get("tg.scheduler", False):
        scheduler._stop_scheduler()
        log.info("Scheduler stopped")
        
old_object_trail = _cputil.get_object_trail

# hang on to object trail to use it to find an app root if need be
def get_object_trail(object_path=None):
    trail = old_object_trail(object_path)
    try:
        cherrypy.request.object_trail = trail
    except AttributeError:
        pass
    return trail

_cputil.get_object_trail = get_object_trail

class SimpleWSGIServer(CherryPyWSGIServer):
    """A WSGI server that accepts a WSGI application as a parameter."""
    RequestHandlerClass = CPHTTPRequest
    
    def __init__(self):
        conf = cherrypy.config.get
        wsgi_app = wsgiApp
        if conf('server.environment') == 'development':
            try:
                from paste.evalexception.middleware import EvalException
            except ImportError:
                pass
            else:
                wsgi_app = EvalException(wsgi_app, global_conf={})
                cherrypy.config.update({'server.throw_errors':True})
        bind_addr = (conf("server.socket_host"), conf("server.socket_port"))
        CherryPyWSGIServer.__init__(self, bind_addr, wsgi_app,
                                    conf("server.thread_pool"),
                                    conf("server.socket_host"),
                                    request_queue_size = conf(
                                        "server.socket_queue_size"),
                                    )

def start_server(root):
    cherrypy.root = root
    if config.get("tg.fancy_exception", False):
        cherrypy.server.start(server=SimpleWSGIServer())
    else:
        cherrypy.server.start()

if startTurboGears not in cherrypy.server.on_start_server_list:
    cherrypy.server.on_start_server_list.append(startTurboGears)

if stopTurboGears not in cherrypy.server.on_stop_server_list:
    cherrypy.server.on_stop_server_list.append(stopTurboGears)

call_on_startup = []
call_on_shutdown = []



from turbogears import database
from sqlobject import dbconnection
import cherrypy

hub = database.AutoConnectHub("sqlite:///:memory:")

def test_registry():
    "hubs appear in a registry"
    assert hub in database.hub_registry

def test_alwaysTransaction():
    "hub.getConnection always returns a Transaction"
    assert isinstance(hub.getConnection(), dbconnection.Transaction)
    database.end_all()
    assert not isinstance(hub.threadingLocal.connection, dbconnection.Transaction)

class DatabaseStandIn:
    committed = False
    rolled_back = False
    ended = False
    successful_called = False
    
    def __init__(self):
        self.old_commit = database.commit_all
        self.old_rollback = database.rollback_all
        self.old_end = database.end_all
        database.commit_all = self.commit
        database.end_all = self.end
        database.rollback_all = self.rollback
    
    def commit(self):
        self.committed = True
    
    def rollback(self):
        self.rolled_back = True
    
    def end(self):
        self.ended = True
        self.old_end()
        
    def successful(self):
        self.successful_called = True
    
    def failure(self):
        self.failure_called = True
        raise "Oh my!"
        
    def redirect(self):
        self.redirect_called = True
        raise cherrypy.HTTPRedirect("/")
    
    def restore(self):
        database.commit_all = self.old_commit
        database.rollback_all = self.old_rollback
        database.end_all = self.end

def test_good_transaction():
    "successful runs automatically commit"
    dsi = DatabaseStandIn()
    database.run_with_transaction(dsi.successful)
    dsi.restore()
    assert dsi.successful_called
    assert dsi.committed
    assert dsi.ended
    
def test_bad_transaction():
    "failed runs automatically rollback"
    dsi = DatabaseStandIn()
    try:
        database.run_with_transaction(dsi.failure)
        dsi.restore()
        assert False, "exception should have been raised"
    except:
        pass
    dsi.restore()
    assert dsi.failure_called
    assert dsi.rolled_back
    assert dsi.ended

def test_redirection():
    "Redirects count as successful runs, not failures"
    dsi = DatabaseStandIn()
    try:
        database.run_with_transaction(dsi.redirect)
    except cherrypy.HTTPRedirect:
        pass
    dsi.restore()
    assert dsi.redirect_called
    assert dsi.committed
    assert dsi.ended

def test_so_to_dict():
    from sqlobject import IntCol
    from sqlobject.inheritance import InheritableSQLObject

    class Parent(InheritableSQLObject):
        _connection = hub
        a   = IntCol()

    class Child(Parent):
        _connection = hub
        b   = IntCol()

    Parent.createTable()
    Child.createTable()
    p =  Parent(a=1)
    c =  Child(a=1, b=2)

    p_dict = database.so_to_dict(p)
    assert p_dict['a'] == 1

    c_dict = database.so_to_dict(c)
    assert c_dict['a'] == 1
    assert c_dict['b'] == 2
    assert None == c_dict.get('childName', None)
    assert None == p_dict.get('childName', None)



import sys
from itertools import izip, repeat, chain as ichain

from dispatch import strategy, functions


class MultiorderGenericFunction(functions.GenericFunction):

    """Generic function allowing a priori method ordering."""

    def __init__(self, func):
        functions.GenericFunction.__init__(self, func)
        self.order_when = []
        self.order_around = []

    def when(self, cond, order=0):
        if order not in self.order_when:
            self.order_when.append(order)
            self.order_when.sort()
        return self._decorate(cond, "primary%d" % order)

    def around(self, cond, order=0):
        if order not in self.order_around:
            self.order_around.append(order)
            self.order_around.sort()
        return self._decorate(cond, "around%d" % order)

    # Based on dispatch.functions.GenericFunction.combine
    def combine(self, cases):
        strict = [strategy.ordered_signatures,strategy.safe_methods]
        loose  = [strategy.ordered_signatures,strategy.all_methods]

        primary_names = ['primary%d' % order for order in self.order_when]
        around_names = ['around%d' % order for order in self.order_around]

        cases = strategy.separate_qualifiers(
            cases,
            before = loose, after =loose,
            **dict(izip(ichain(primary_names, around_names), repeat(strict)))
        )

        primary = strategy.method_chain(ichain(
                    *[cases.get(primary, []) for primary in primary_names]))

        if cases.get('after') or cases.get('before'):

            befores = strategy.method_list(cases.get('before',[]))
            afters = strategy.method_list(list(cases.get('after',[]))[::-1])

            def chain(*args,**kw):
                for tmp in befores(*args,**kw): pass  # toss return values
                result = primary(*args,**kw)
                for tmp in afters(*args,**kw): pass  # toss return values
                return result

        else:
            chain = primary

        if (self.order_around):
            chain = strategy.method_chain(ichain(*([cases.get(around, [])
                                    for around in around_names] + [[chain]])))

        return chain


def getter(var):
    """Create an accessor for given variable."""
    frame = sys._getframe(1)
    return lambda: var in frame.f_locals and frame.f_locals[var] or \
                                             frame.f_globals[var]

__all__ = ["MultiorderGenericFunction", "getter", ]



import types
import inspect
import logging
import unittest
import cStringIO as StringIO
import Cookie

import cherrypy
try:
    import sqlobject
    from sqlobject.inheritance import InheritableSQLObject
except ImportError:
    sqlobject = None

try:
    import sqlalchemy
except ImportError:
    sqlalchemy = None

from cherrypy import _cphttptools

from turbogears import database, controllers, startup, validators, config, \
                       update_config
from turbogears.util import get_model

import os
from os.path import *

cwd = os.getcwd()

# For clean tests, remove all compiled Kid templates
for w in os.walk(cwd):
    if w[0] != '.svn':
        for f in w[2]:
            if f.endswith('.kid'):
                f = join(w[0], f[:-3] + 'pyc')
                if exists(f):
                    os.remove(f)

# Override config of all applications with test.cfg
if exists(join(cwd, "test.cfg")):
    modulename = None
    for w in os.walk(cwd):
        if w[0].endswith("config"):
            config_dir = w[0].replace(cwd, "")[1:]
            modulename = "%s.app" % config_dir.replace(os.sep, ".")
            break
    update_config(configfile=join(cwd, "test.cfg"), modulename=modulename)
else:
    database.set_db_uri("sqlite:///:memory:")

config.update({"global" : {"tg.new_style_logging" : True}})
config.update({"global" : {"autoreload.on" : False}})

def start_cp():
    if not config.get("cherrypy_started", False):
        cherrypy.server.start(serverClass=None, initOnly=True)
        config.update({"cherrypy_started" : True})

test_user = None

def set_identity_user(user):
    "Setup a user which will be used to configure request's identity."
    global test_user
    test_user = user

def attach_identity(req):
    from turbogears.identity import current_provider
    if config.get("identity.on", False):
        if test_user:
            id = current_provider.authenticated_identity(test_user)
        else:
            id = current_provider.anonymous_identity()
        req.identity = id

def create_request(request, method="GET", protocol="HTTP/1.1",
    headers={}, rfile=None, clientAddress="127.0.0.1",
    remoteHost="localhost", scheme="http"):
    start_cp()
    if not rfile:
        rfile = StringIO.StringIO("")
    if type(headers) != dict:
        headerList = headers
    else:
        headerList = [(key, value) for key, value in headers.items()]
    headerList.append(("Host", "localhost"))
    if not hasattr(cherrypy.root, "started"):
        startup.startTurboGears()
        cherrypy.root.started = True
    req = _cphttptools.Request(clientAddress, 80, remoteHost, scheme)
    cherrypy.serving.request = req
    attach_identity(req)
    cherrypy.serving.response = _cphttptools.Response()
    req.run(" ".join((method, request, protocol)), headerList, rfile)

createRequest = create_request


class BrowsingSession(object):
    def __init__(self):
        self.visit = None
        self.response, self.status = None, None
        self.cookie = Cookie.SimpleCookie()

    def goto(self, *args, **kwargs):
        if self.cookie:
            headers = kwargs.setdefault('headers', {})
            headers['Cookie'] = self.cookie.output()
        create_request(*args, **kwargs)
        self.response = cherrypy.response.body[0]
        self.status = cherrypy.response.status
        if cherrypy.response.simple_cookie:
            self.cookie.update(cherrypy.response.simple_cookie)


def _return_directly(output, *args):
    return output

class DummySession:
    session_storage = dict
    to_be_loaded = None

class DummyRequest:
    "A very simple dummy request."
    remote_host = "127.0.0.1"

    def __init__(self, method='GET', path='/', headers=None):
        self.headers = headers or {}
        self.method = method
        self.path = path
        self.base = ''
        self._session = DummySession()
    def purge__(self):
        pass

def call(method, *args, **kw):
    start_cp()
    output, response = call_with_request(method, DummyRequest(), *args, **kw)
    return output

def call_with_request(method, request, *args, **kw):
    "More fine-grained version of call method, allowing to use request/response."
    orig_proc_output = controllers._process_output
    controllers._process_output = _return_directly
    cherrypy.serving.response = _cphttptools.Response()
    cherrypy.serving.request = request
    if not hasattr(request, "identity"):
        attach_identity(request)
    output = None
    try:
        output = method(*args, **kw)
    finally:
        del cherrypy.serving.request
        controllers._process_output = orig_proc_output
    response = cherrypy.serving.response
    return output, response


class DBTest(unittest.TestCase):
    model = None

    def setUp(self):
        if not self.model:
            self.model = get_model()
            if not self.model:
                raise "Unable to run database tests without a model"

        for item in self.model.__dict__.values():
            if isinstance(item, types.TypeType) and issubclass(item,
                sqlobject.SQLObject) and item != sqlobject.SQLObject \
                and item != InheritableSQLObject:
                item.createTable(ifNotExists=True)

    def tearDown(self):
        database.rollback_all()
        for item in self.model.__dict__.values():
            if isinstance(item, types.TypeType) and issubclass(item,
                sqlobject.SQLObject) and item != sqlobject.SQLObject \
                and item != InheritableSQLObject:
                item.dropTable(ifExists=True)

def reset_cp():
    cherrypy.root = None

def catch_validation_errors(widget, value):
    """ Catches and unpacks validation errors. For testing purposes. """
    errors = {}
    try:
        value = widget.validate(value)
    except validators.Invalid, e:
        if hasattr(e, 'unpack_errors'):
            errors = e.unpack_errors()
        else:
            errors = e
    return value, errors

class MemoryListHandler(logging.Handler):

    def __init__(self):
        logging.Handler.__init__(self, level=logging.DEBUG)
        self.log = []

    def emit(self, record):
        print "Got record: %s" % record
        print "formatted as: %s" % self.format(record)
        self.log.append(self.format(record))

    def print_log(self):
        print "\n".join(self.log)
        self.log = []

    def get_log(self):
        log = self.log
        self.log = []
        return log

_memhandler = MemoryListHandler()
_currentcat = None

def capture_log(category):
    """Category can either be a single category (a string like 'foo.bar')
    or a list of them. You <em>must</em> call print_log() to reset when
    you're done."""
    global _currentcat
    assert not _currentcat
    if not isinstance(category, list) and not isinstance(category, tuple):
        category = [category]
    _currentcat = category
    for cat in category:
        log = logging.getLogger(cat)
        log.setLevel(logging.DEBUG)
        log.addHandler(_memhandler)

def _reset_logging():
    """Manages the resetting of the loggers"""
    global _currentcat
    if not _currentcat:
        return
    for cat in _currentcat:
        log = logging.getLogger(cat)
        log.removeHandler(_memhandler)
    _currentcat = None

def print_log():
    """Prints the log captured by capture_log to stdout, resets that log
    and resets the temporarily added handlers."""
    _reset_logging()
    _memhandler.print_log()

def get_log():
    """Returns the list of log messages captured by capture_log,
    resets that log and resets the temporarily added handlers."""
    _reset_logging()
    return _memhandler.get_log()

def sqlalchemy_cleanup():
    database._engine = None
    sqlalchemy.clear_mappers()
    database.metadata.clear()
    database.metadata.dispose()

__all__ = ["create_request", "call", "DBTest", "createRequest",
    "attach_identity", "set_identity_user",
    "capture_log", "print_log", "get_log", "sqlalchemy_cleanup"]



#!/usr/bin/env python
# -*- coding: ascii -*-
#
# $Id: pythondoc.py 2711 2006-04-06 07:16:21Z fredrik $
# pythondoc documentation generator
#
# history:
# 2003-10-19 fl   first preview release (2.0a1)
# 2003-10-19 fl   fix HTML in descriptor tags, 1.5.2 tweaks, etc (2.0a2)
# 2003-10-20 fl   added encoding support, default HTML generator, etc (2.0a3)
# 2003-10-21 fl   fixed some 1.5.2 issues, etc (2.0b1)
# 2003-10-22 fl   HTML tweaks, pluggable output generators, etc (2.0b2)
# 2003-10-23 fl   fixed encoding, added @author, @version, @since etc
# 2003-10-24 fl   disable XML output by default
# 2003-10-25 fl   moved info properties into an 'info' element
# 2003-10-26 fl   expand wildcards on windows (2.0b3)
# 2003-10-30 fl   added support for RISC OS
# 2003-10-31 fl   (experimental) support module-level comments
# 2003-11-01 fl   minor HTML tweaks (2.0b4)
# 2003-11-03 fl   pythondoc 2.0 final
# 2003-11-15 fl   added support for inline @link/@linkplain tags (2.1b1)
# 2003-11-20 fl   fixed class attribute parsing bug
# 2004-03-27 fl   handle multiple single-line methods
# 2004-09-01 fl   support Python 2.4 decorators (2.1b2)
# 2004-09-21 fl   fixed output filename for "pythondoc ."
# 2005-03-25 fl   added docstring extraction for classes and methods (2.1b3)
# 2005-06-18 fl   fixed correct HTML output when using ElementTree 1.3 (2.1b4)
# 2005-12-23 fl   use xml.etree where available
# 2006-04-04 fl   refactored comment parser code; added -s support (2.1b5)
# 2006-04-06 fl   handle multiple params in docstrings correctly (2.1b6)
#
# Copyright (c) 2002-2006 by Fredrik Lundh.
#

##
# This is the PythonDoc tool.  This tool parses Python source files
# and generates API descriptions in XML and HTML.
# <p>
# For more information on the PythonDoc tool and the markup format, see
# <a href="http://effbot.org/zone/pythondoc.htm">the PythonDoc page</a>
# at <a href="http://effbot.org/">effbot.org</a>.
##

# --------------------------------------------------------------------
# Software License
# --------------------------------------------------------------------
#
# Copyright (c) 2002-2006 by Fredrik Lundh
#
# By obtaining, using, and/or copying this software and/or its
# associated documentation, you agree that you have read, understood,
# and will comply with the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and
# its associated documentation for any purpose and without fee is
# hereby granted, provided that the above copyright notice appears in
# all copies, and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# Secret Labs AB or the author not be used in advertising or publicity
# pertaining to distribution of the software without specific, written
# prior permission.
#
# SECRET LABS AB AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
# TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANT-
# ABILITY AND FITNESS.  IN NO EVENT SHALL SECRET LABS AB OR THE AUTHOR
# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#
# --------------------------------------------------------------------

# to do in later releases:
#
# TODO: test this release under 1.5.2 !
# TODO: better rendering of constructors/package modules
# TODO: check @param names against @def/define tags
# TODO: support recursive parsing (-R)
# TODO: warn for tags that doesn't make sense for a given target type
# TODO: HTML output localization (the %s module, returns, raises, etc)
# TODO: make compactHTML generate an element tree instead of raw HTML
#
# nice to have, maybe:
#
# IDEA: support multiple output handlers (multiple -O statements);
#       make -x an alias for -Oxml
# IDEA: make pythondoc self-contained (include stub element implementation)

VERSION_DATE = "2.1b6-20060406"
VERSION = VERSION_DATE.split("-")[0]

COPYRIGHT = "(c) 2002-2006 by Fredrik Lundh"

# explicitly import site (for pythonworks exe builder)
import site

# stuff we use in this module
import glob, os, re, string, sys, tokenize

# make sure elementtree is available
try:
    try:
        # note: HTMLTreeBuilder isn't included in Python 2.5 alpha 1
        from xml.etree import ElementTree, HTMLTreeBuilder
    except ImportError:
        from elementtree import ElementTree, HTMLTreeBuilder
except ImportError:
    raise RuntimeError(
        "PythonDoc %s requires ElementTree 1.1 or later "
        "(available from http://effbot.org/downloads)." % VERSION
        )

# extension separator (not all systems use a period)
try:
    EXTSEP = os.extsep
except AttributeError:
    EXTSEP = "."

##
# Debug level.  The higher the value, the more junk you'll see on
# standard output.
# <p>
# You can use the <b>-V</b> option to <b>pythondoc</b> to increase
# the debug level.

DEBUG = 0

##
# Whitespace tokens.  These are ignored when the parser is scanning
# for a subject.

WHITESPACE_TOKEN = (
    tokenize.NL, tokenize.NEWLINE, tokenize.DEDENT, tokenize.INDENT
    )

##
# Default encoding.  To override this for a module, put a "coding"
# directive in your Python module (see PEP 263 for details).

ENCODING = "iso-8859-1"

##
# Known tags.  The parser generates warnings for tags that are not in
# this list, but it still copies them to the XML infoset.

TAGS = (
    "def", "defreturn",
    "param", "keyparam",
    "return",
    "throws", "exception",
    # javadoc tags not used by the standard generator
    "author", "deprecated", "see", "since", "version"
    )

##
# (Helper) Combines filename prefix with extension part.
#
# @param prefix Filename prefix.
# @param ext Extension string, including a leading period.  The
#    period is replaced with a platform-specific separator, if
#    necessary.
# @return The combined name.

def joinext(prefix, ext):
    assert ext[0] == "." # require leading separator, to match os.path.splitext
    return prefix + EXTSEP + ext[1:]

##
# (Helper) Extracts block tags from a PythonDoc comment.
#
# @param comment Comment text.
# @return A list of (lineno, tag, text) tuples, where the tag is None
#     for the initial description.
# @defreturn List of tuples.

def gettags(comment):

    tags = []

    tag = None
    tag_lineno = lineno = 0
    tag_text = []

    for line in comment:
        if line[:1] == "@":
            tags.append((tag_lineno, tag, string.join(tag_text, "\n")))
            line = string.split(line, " ", 1)
            tag = line[0][1:]
            if len(line) > 1:
                tag_text = [line[1]]
            else:
                tag_text = []
            tag_lineno = lineno
        else:
            tag_text.append(line)
        lineno = lineno + 1

    tags.append((tag_lineno, tag, string.join(tag_text, "\n")))

    return tags

##
# (Helper) Flattens an element tree, returning only the text contents.
#
# @param elem An element tree.
# @return A text string.
# @defreturn String.

def flatten(elem):
    text = elem.text or ""
    for e in elem:
        text += flatten(e)
        if e.tail:
            text += e.tail
    return text

##
# (Helper) Extracts summary from a PythonDoc comment.  This function
# gets the first complete sentence from the description string.
#
# @param description An element containing the description.
# @return A summary string.
# @defreturn String.

def getsummary(description):

    description = flatten(description)

    # extract the first sentence from the description
    m = re.search("(?s)(.+?\.)\s", description + " ")
    if m:
        return m.group(1)

    return description # sorry

##
# (Helper) Parses HTML descriptor text into an XHTML structure.
#
# @param parser Parser instance (provides a warning method).
# @param text Text fragment.
# @return An element tree containing XHTML data.
# @defreturn Element.

def parsehtml(parser, tag, text, lineno):

    # transcode
    if parser.encoding != "ascii":
        try:
            text = unicode(text, parser.encoding)
        except NameError:
            pass # 1.5.2

    # process inline links (@link, @linkplain)
    # note that links are replaced with <a href='link:...> elements;
    # the href's are resolved in a later step
    if "{" in text:
        def fixlink(m, parser=parser, lineno=lineno):
            linkdef = string.split(m.group(1), None, 2)
            if len(linkdef) == 2:
                # default text is same as last part of target name
                href = linkdef[1]
                if href[:1] == "#":
                    href = href[1:]
                linkdef.append(string.split(href, ".")[-1])
            if len(linkdef) < 3 or linkdef[0] not in ("@link", "@linkplain"):
                parser.warning(
                    (lineno, 0),
                    "malformed @link near this line",
                    )
                return m.group(0)
            type, href, text = linkdef
            href = "link:" + html_encode(href)
            if type == "@link":
                return "<a href='%s' class='link'><b>%s</b></a>" % (href, text)
            else:
                return "<a href='%s' class='linkplain'>%s</a>" % (href, text)
        text = re.sub("\{(@link[^}]+)\}", fixlink, text)

    if "<" not in text and "&" not in text:
        # plain text
        elem = ElementTree.Element(tag)
        elem.text = string.strip(text)
        return elem

    p = HTMLTreeBuilder.TreeBuilder()
    ix = 0
    try:
        p.feed("<%s>" % tag)
        p.feed("<p>") # make sure everything's wrapped in a paragraph tag
        # feed line by line
        for line in string.split(text, "\n"):
            p.feed(line + "\n")
            ix = ix + 1
        p.feed("</%s>" % tag)
        tree = p.close()
    except:
        parser.warning(
            (lineno+ix, 0),
            "HTML parser error near this line (%s)",
            sys.exc_value
            )
        return ElementTree.Element("p")

    return tree


##
# (Helper) Parses a PythonDoc comment into an PythonDoc info structure.
#
# @param parser Parser instance (provides a warning method).
# @param lineno Line number where this comment starts.
# @param comment A list of text line making up the comment.
# @param dedent If true, strip leading whitespace from all comment
#     lines except the first one.
# @return An element tree containing XHTML data.
# @defreturn Element.

def parsecomment(parser, lineno, comment, dedent=0):

    subject_info = ElementTree.Element("info")

    # untabify
    for ix in range(len(comment)):
        comment[ix] = string.expandtabs(comment[ix])

    if dedent:
        margin = None
        for ix in range(1, len(comment)):
            s = string.lstrip(comment[ix])
            if not s:
                continue
            m = len(comment[ix]) - len(s)
            if margin is None:
                margin = m
            else:
                margin = min(m, margin)
        if margin:
            for ix in range(1, len(comment)):
                comment[ix] = comment[ix][margin:]

    for ix, tag, text in gettags(comment):

        pos = lineno + ix + 1, 0

        # check tag name
        if tag is None:
            tag = "description"
        else:
            if tag not in TAGS:
                parser.warning(
                    pos,
                    "unknown tag in description: @%s", tag
                    )
            if tag in ("throws", "exception"):
                tag = "exception" # PythonDoc extension

        # deal with "named" tags
        if tag in ("param", "keyparam", "exception"):
            text = string.split(text, " ", 1)
            name = text[0]
            if len(text) > 1:
                text = string.lstrip(text[1])
            else:
                text = ""
        else:
            name = None

        tag_elem = parsehtml(parser, tag, text, pos[0])

        # generate summaries
        if tag == "description":
            summary = getsummary(tag_elem)
            if summary:
                elem = ElementTree.SubElement(subject_info, "summary")
                elem.text = summary

        subject_info.append(tag_elem)

        if name:
            tag_elem.set("name", name)

    return subject_info

##
# Module parser.
# <p>
# This class implements the PythonDoc source code scanner.  It reads
# source code from a file or a file-like object, and builds an element
# tree with information about the module.
# <p>
# Note that the constructor only sets things up for parsing.  Use the
# {@link ModuleParser.parse} method to parse the file.  Or for
# convenience, use the {@link parse} function to create a parser
# object and parse a given file.
#
# @param file Name of the module source file, or a file object.  If a
#    file object is used, it must provide a <b>name</b> attribute and
#    a <b>readline</b> method.
# @param prefix Optional name prefix.  If given, this is prepended to
#    the module name.  For example, if the prefix is set to "prefix"
#    and the module filename is "name.py", the module is assumed to
#    contain the "prefix.name" namespace.

class ModuleParser:

    ##
    # Module name.

    name = None

    def __init__(self, file, prefix=None):
        if hasattr(file, "readline"):
            self.file = file
            self.filename = file.name
        else:
            self.file = None
            self.filename = file
        name = os.path.splitext(os.path.basename(self.filename))[0]
        if prefix and prefix != ".":
            name = prefix + "." + name
        self.name = name
        self.stack = [
            ElementTree.Element(
                "module",
                name=name, filename=self.filename
                )
            ]
        self.indent = 0
        self.scope = [] # list of (indent, tag, name, ...) tuples
        self.handler = self.look_for_encoding
        self.encoding = ENCODING

    ##
    # Parses the file.
    #
    # @keyparam docstring If true, look for markup in docstrings.
    # @return An element tree containing information about the module.
    # @defreturn Element.
    # @exception IOError If the file could not be opened.

    def parse(self, docstring=0):
        if self.file is None:
            file = open(self.filename)
        else:
            file = self.file
        try:
            tokenize.tokenize(file.readline, self.handle_token)
        except tokenize.TokenError, v:
            message, lineno = v
            self.warning(lineno, "exception in tokenizer: %s", message)
        if len(self.stack) != 1:
            pass # FIXME: print warning?
        tree = self.stack[0] # may be incomplete
        # fixup internal links
        # 1) find all named elements
        elems = {}
        for elem in tree.getiterator():
            name = elem.get("name")
            if name:
                elems[name] = elem
        # 2) find all link anchors
        for elem in tree.getiterator("a"):
            href = elem.get("href")
            if href[:5] == "link:":
                # FIXME: add support for external links
                href = href[5:]
                if href[:1] == "#":
                    href = href[1:]
                target = elems.get(self.name + "." + href)
                if target:
                    href = "#" + target.get("name") + "-" + target.tag
                    elem.set("href", href)
        if docstring:
            # look for markup in docstrings
            for info in tree.getiterator("info"):
                docstring = info.findtext("docstring")
                if not docstring:
                    continue
                comment = docstring.split("\n")
                newinfo = parsecomment(self, 0, comment, dedent=1)
                for elem in info:
                    if newinfo.find(elem.tag) is None:
                        newinfo.append(elem)
                info[:] = newinfo
        return tree

    ##
    # Prints a warning message to standard output.
    #
    # @param position A (line, column) tuple.  The column can be set
    #     to None if not known (or not relevant).
    # @param format Message or format string.
    # @param *args Optional arguments.

    def warning(self, position, format, *args):
        line, column = position
        message = "%s:%d: WARNING: %s" % (self.filename, line, format % args)
        sys.stderr.write(message)
        sys.stderr.write("\n")

    ##
    # Dispatches tokens to the current handler.  Each handler should
    # return the handler to call for the next token.
    # <p>
    # This method also handles indentation and dedentation tokens,
    # and manages the scope stack.

    def handle_token(self, *args):
        # dispatch incoming tokens to the current handler
        if DEBUG > 1:
            print self.handler.im_func.func_name, self.indent,
            print tokenize.tok_name[args[0]], repr(args[1])
        if args[0] == tokenize.DEDENT:
            self.indent = self.indent - 1
            while self.scope and self.scope[-1][0] >= self.indent:
                del self.scope[-1]
                del self.stack[-1]
        self.handler = apply(self.handler, args)
        if args[0] == tokenize.INDENT:
            self.indent = self.indent + 1

    ##
    # (Token handler) Scans for encoding directive.

    def look_for_encoding(self, type, token, start, end, line):
        if type == tokenize.COMMENT:
            if string.rstrip(token) == "##":
                return self.look_for_pythondoc(type, token, start, end, line)
            m = re.search("coding[:=]\s*([-_.\w]+)", token)
            if m:
                self.encoding = m.group(1)
                return self.look_for_pythondoc
        if start[0] > 2:
            return self.look_for_pythondoc
        return self.look_for_encoding

    ##
    # (Token handler) Scans for PythonDoc comments.

    def look_for_pythondoc(self, type, token, start, end, line):
        if type == tokenize.COMMENT and string.rstrip(token) == "##":
            # found a comment: set things up for comment processing
            self.comment_start = start
            self.comment = []
            return self.process_comment_body
        else:
            # deal with "bare" subjects
            if token == "def" or token == "class":
                self.subject_indent = self.indent
                self.subject_parens = 0
                self.subject_start = self.comment_start = None
                self.subject = []
                return self.process_subject(type, token, start, end, line)
            return self.look_for_pythondoc

    ##
    # (Token handler) Processes a comment body.  This handler adds
    # comment lines to the current comment.

    def process_comment_body(self, type, token, start, end, line):
        if type == tokenize.COMMENT:
            if start[1] != self.comment_start[1]:
                self.warning(
                    start,
                    "comment line should be aligned with marker"
                    )
            line = string.rstrip(token)
            if line == "##":
                # handle module comments (experimental)
                # FIXME: add more consistency checks?
                if self.stack[0].find("info") is not None:
                    self.warning(
                        self.comment_start,
                        "multiple module comments are not allowed"
                        )
                    # FIXME: ignore additional comments?
                self.process_subject_info(None, self.stack[0])
                return self.look_for_pythondoc
            elif line[:2] == "# ":
                line = line[2:]
            elif line[:1] == "#":
                line = line[1:]
            self.comment.append(line)
        else:
            if not self.comment:
                self.warning(
                    self.comment_start,
                    "found pythondoc marker but no comment body"
                    )
                return self.look_for_pythondoc
            self.subject_start = None
            self.subject = []
            if type != tokenize.NL:
                return self.process_subject(type, token, start, end, line)
            return self.process_subject # end of comment
        return self.process_comment_body

    ##
    # (Token handler) Processes the comment subject.  The subject can
    # be either a plain variable, or a function/method or class
    # definition.
    # <p>
    # This method is also used to process "bare" subjects; that is,
    # functions, methods, and classes that don't have PythonDoc
    # markup.  In that case, the comment_start variable is set to
    # None.

    def process_subject(self, type, token, start, end, line):
        # got an item; deal with it
        if self.subject:
            # method/function/class definition
            definition = self.subject[0] in ("def", "class")
            if definition:
                if type not in WHITESPACE_TOKEN:
                    if token == "(":
                        self.subject_parens = self.subject_parens + 1
                    elif token == ")":
                        self.subject_parens = self.subject_parens - 1
                if self.subject_parens or token != ":":
                    self.subject.append(token)
                    return self.process_subject
            else:
                # simple assignment
                if token != "=":
                    self.warning(
                        self.subject_start,
                        "bad subject %s; ignoring description",
                        repr(self.subject[0])
                        )
                    # might be a pythondoc marker; pass it to the scanner
                    return self.look_for_pythondoc(
                        type, token, start, end, line
                        )
                # FIXME: keep adding stuff until end of expression
        else:
            if type in WHITESPACE_TOKEN:
                return self.process_subject
            if type == tokenize.COMMENT:
                self.warning(
                    start,
                    "comment between description and subject; " +
                    "ignoring description"
                    )
                # might be a pythondoc marker; pass it to the scanner
                return self.look_for_pythondoc(
                    type, token, start, end, line
                    )
            # FIXME: check token type!
            # the @ token type is currently tokenize.ERRORTOKEN; hopefully
            # this will change before 2.4 final
            if token == "@":
                self.decorator_parens = 0
                return self.skip_decorator
            self.subject_start = start
            self.subject.append(token)
            if token in ("def", "class"):
                # handle single-line subjects
                while self.scope and self.scope[-1][0] >= self.indent:
                    self.scope.pop()
                    self.stack.pop()
                self.subject_indent = self.indent
                self.subject_parens = 0
            return self.process_subject

        # check if this is a method or a function
        method = self.scope and self.scope[-1][1] == "class"

        # calculate fully qualified subject name
        name = [self.name]
        for s in self.scope:
            name.append(s[2])
        if definition:
            name.append(self.subject[1])
        else:
            name.append(self.subject[0])

        # calculate subject definition statement
        statement = []
        for part in self.subject:
            if part in ("class", "def"):
                continue
            statement.append(part)
            if part == ",":
                statement.append(" ")
        if self.subject[0] == "def" and method:
            # ignore the first argument for methods
            # 'name', '(', 'self', ',', ' ', ...)
            del statement[2:min(5, len(statement)-1)]
        statement = string.join(statement, "")

        # create subject element
        if self.subject[0] == "class":
            subject_elem = ElementTree.Element("class")
        elif self.subject[0] == "def":
            if method:
                subject_elem = ElementTree.Element("method")
            else:
                subject_elem = ElementTree.Element("function")
        else:
            subject_elem = ElementTree.Element("variable")

        self.stack[-1].append(subject_elem)

        # add new subject to the scope and element stacks
        if definition:
            self.scope.append((self.subject_indent,) + tuple(self.subject))
            self.stack.append(subject_elem)

        subject_info = self.process_subject_info(name, subject_elem)

        # add local name to info
        elem = ElementTree.Element("name")
        elem.text = name[-1]

        subject_info.insert(0, elem)

        name = string.join(name, ".")

        subject_elem.set("name", name)
        subject_elem.set("lineno", str(self.subject_start[0]))

        if subject_info.find("def") is None and statement:
            # add subject definition (unless specified in comment)
            elem = ElementTree.Element("def")
            elem.text = statement
            # add to front, to make the XML easier to read
            subject_info.insert(0, elem)

        if definition:
            return self.look_for_docstring(type, token, start, end, line)
        else:
            return self.look_for_pythondoc(type, token, start, end, line)

    ##
    # (Token handler) Skips a decorator.

    def skip_decorator(self, type, token, start, end, line):
        if token == "(":
            self.decorator_parens = self.decorator_parens + 1
        elif token == ")":
            self.decorator_parens = self.decorator_parens - 1
        if self.decorator_parens or type != tokenize.NEWLINE:
            return self.skip_decorator
        return self.process_subject

    ##
    # (Token handler helper) Processes a PythonDoc comment.  This
    # method creates an "info" element based on the current comment,
    # and attaches it to the current subject element.
    #
    # @param subject_name Subject name (or None if the name is not known).
    # @param subject_elem The current subject element.
    # @return The info element.  Note that this element has already
    #     been attached to the subject element.
    # @defreturn Element

    def process_subject_info(self, subject_name, subject_elem):

        # process pythondoc comment (if any)
        if self.comment_start:
            subject_info = parsecomment(
                self, self.comment_start[0], self.comment
                )
        else:
            subject_info = ElementTree.Element("info")

        subject_elem.append(subject_info)

        if DEBUG:
            if subject_name:
                subject_name = string.join(subject_name, ".")
            else:
                subject_name = "<module>"
            print "---", subject_name
            prefix = "   " * len(self.scope)
            for line in self.comment:
                print prefix + line

        return subject_info

    ##
    # (Token handler) Look for docstring inside a definition.

    def look_for_docstring(self, type, token, start, end, line):
        if type in WHITESPACE_TOKEN or token == ":":
            return self.look_for_docstring
        if type == tokenize.STRING:
            subject_elem = self.stack[-1]
            subject_info = subject_elem.find("info")
            if subject_info is not None:
                elem = ElementTree.SubElement(subject_info, "docstring")
                # FIXME: add string sanity check here, before doing eval
                elem.text = eval(token)
        return self.skip_subject_body(type, token, start, end, line)

    ##
    # (Token handler) Skips over the subject body.

    def skip_subject_body(self, type, token, start, end, line):
        # for now, just hand control back to the pythondoc scanner,
        # and let it skip over the subject body while looking for the
        # next marker.
        return self.look_for_pythondoc(type, token, start, end, line)

##
# Parses a module.
# <p>
# This function creates a {@link #ModuleParser} instance, and uses it
# to parse the given file.  For details, see {@linkplain #ModuleParser
# the <b>ModuleParser</b> documentation}.
#
# @param file Name of the module source file, or a file object.
# @param prefix Optional name prefix.
# @keyparam docstring If true, look for markup in docstrings.
# @return An element tree containing the module description.
# @defreturn Element.
# @exception IOError If the file could not be found, or could not
#    be opened for reading.

def parse(file, prefix=None, docstring=0):
    m = ModuleParser(file, prefix)
    return m.parse(docstring=docstring)

# --------------------------------------------------------------------
# default formatter

if sys.version[:3] == "1.5":
    _escape = re.compile(r"[&<>\"\x80-\xff]") # 1.5.2
else:
    _escape = re.compile(eval(r'u"[&<>\"\u0080-\uffff]"'))

_escape_map = {
    "&": "&amp;",
    "<": "&lt;",
    ">": "&gt;",
    '"': "&quot;",
}

##
# Encodes reserved HTML characters and non-ASCII characters as HTML
# character references.
#
# @def html_encode(text)
# @param text Source text.
# @return An encoded string.

def html_encode(text, pattern=_escape):
    if not text:
        return ""
    def escape_entities(m, map=_escape_map):
        char = m.group()
        text = map.get(char)
        if text is None:
            text = "&#%d;" % ord(char)
        return text
    text = pattern.sub(escape_entities, text)
    try:
        return text.encode("ascii")
    except AttributeError:
        return text # 1.5.2

##
# Compact HTML formatter.  This formatter turns a module XML
# description into a minimal HTML document.
# <p>
# This formatter supports the following options:
# </p>
# <dl>
# <dt><b>-Dstyle</b>=URL</dt>
# <dd>Stylesheet URL.  If this option is present, the formatter adds a
# stylesheet &lt;link&gt; element to the HTML output.</dd>
# <dt><b>-Dzone</b></dt>
# <dd>Generate effbot.org zone documents.</dd>
# </dl>
#
# @param options Options dictionary.

class CompactHTML:

    def __init__(self, options=None):
        self.options = options or {}

    ##
    # Writes an element containing some text (plain or formatted).
    #
    # @param elem Element.
    # @param compact If true, try to minimize the amount of vertical
    #     padding.

    def writetext(self, elem, compact=0):
        if len(elem):
            if compact and len(elem) == 1 and elem[0].tag == "p":
                elem = elem[0]
                self.file.write(html_encode(elem.text))
                for e in elem:
                    ElementTree.ElementTree(e).write(self.file)
                self.file.write(html_encode(elem.tail))
            else:
                for e in elem:
                    ElementTree.ElementTree(e).write(self.file)
        elif elem is not None and elem.text:
            if compact:
                self.file.write(html_encode(elem.text))
            else:
                self.file.write("<p>%s</p>\n" % html_encode(elem.text))

    ##
    # Writes an object description (the description text, parameters,
    # return values) etc.
    #
    # @param object The object element.
    # @param summary If true, use summary instead of full description.

    def writeobject(self, object, summary=0):
        name = object.get("name")
        info = object.find("info")
        localname = string.split(name, ".")[-1]
        define = info.findtext("def")
        anchor = html_encode(name + "-" + object.tag)
        if object.tag == "class":
            # look for the constructor
            for obj in object:
                if obj.get("name") == name + ".__init__":
                    inf = obj.find("info")
                    define = string.split(inf.findtext("def"), "(", 1)
                    define = localname + "(" + define[1]
                    self.file.write(
                        "<dt><b>%s</b> (class)" % html_encode(define)
                        )
                    break
            else:
                self.file.write(
                    "<dt><b>%s</b> (class) " % html_encode(localname)
                    )
        elif object.tag == "variable":
            self.file.write(
                "<dt><a id='%s' name='%s'><b>%s</b></a> (variable)" % (
                    anchor, anchor, html_encode(localname)
                    )
                )
        else:
            self.file.write(
                "<dt><a id='%s' name='%s'><b>%s</b></a>" % (
                    anchor, anchor, html_encode(define)
                    )
                )
            if object.tag == "function" or object.tag == "method":
                defreturn = info.find("defreturn")
                if defreturn is not None:
                    defreturn = flatten(defreturn)
                    self.file.write(" &rArr; %s" % html_encode(defreturn))
        self.file.write(" [<a href='#%s'>#</a>]</dt>\n" % anchor)
        self.file.write("<dd>\n")
        if summary:
            text = info.findtext("summary")
            if text:
                self.file.write("<p>%s</p>\n" % html_encode(text))
        else:
            self.writetext(info.find("description"))
        param = info.findall("param")
        keyparam = info.findall("keyparam")
        exception = info.findall("exception")
        return_elem = info.find("return")
        if param or keyparam or exception or return_elem != None:
            self.file.write("<dl>\n")
            for p in param + keyparam:
                name = p.get("name")
                if p.tag == "keyparam":
                    name = name + "="
                self.file.write("<dt><i>%s</i></dt>\n" % html_encode(name))
                self.file.write("<dd>\n")
                self.writetext(p, compact=1)
                self.file.write("</dd>\n")
            if return_elem is not None:
                self.file.write("<dt>Returns:</dt>\n")
                self.file.write("<dd>\n")
                self.writetext(return_elem, compact=1)
                self.file.write("</dd>\n")
            for e in exception:
                name = html_encode(e.get("name"))
                self.file.write("<dt>Raises <b>%s</b>:</dt>" % name)
                self.file.write("<dd>\n")
                self.writetext(e, compact=1)
                self.file.write("</dd>\n")
            self.file.write("</dl><br />\n")
        if object.tag == "class" and summary:
            self.file.write(
                "<p>For more information about this class, see "
                "<a href='#%s'><i>The %s Class</i></a>.</p>\n" % (
                    anchor, localname
                    )
                )
        self.file.write("</dd>\n")

    ##
    # Writes a module description to file.
    #
    # @param module A module element tree, as returned by {@link
    # ModuleParser.parse}.
    # @param file Output file name (minus extension).
    # @return If successful, the output filename used to store the
    #     module.
    # @defreturn String or None.

    def save(self, module, file):

        title = "The %s Module" % module.get("name")

        zone = self.options.has_key("zone")

        if zone:
            filename = joinext(file, ".txt")
        else:
            filename = joinext(file, ".html")
        self.file = open(filename, "w")

        if zone:
            # generate zone document
            self.file.write(title + "\n\n")
        else:
            self.file.write(
                "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' "
                "'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>\n"
                )
            self.file.write("<html>\n<head>\n")
            self.file.write(
                "<meta http-equiv='Content-Type' "
                "content='text/html; charset=us-ascii' />\n"
                )
            self.file.write("<title>%s</title>\n" % html_encode(title))
            try:
                style = self.options["style"]
            except KeyError:
                pass
            else:
                self.file.write(
                    "<link rel='stylesheet' href='%s' type='text/css' />\n" %
                    html_encode(style)
                    )
            self.file.write("</head>\n<body>\n")
            self.file.write("<h1>%s</h1>\n" % title)

        # 0) module comments
        info = module.find("info")
        if info is not None:
            self.writetext(info.find("description"))
            self.file.write("<h2>Module Contents</h2>\n")

        # 1) toplevel subjects (including class overviews)
        objects = []
        for object in module:
            info = object.find("info")
            if info is None or info.find("description") is None:
                continue
            if object.tag in ("variable", "function", "class"):
                objects.append(object)
        objects.sort(lambda a, b: cmp(
            string.lower(a.get("name")), string.lower(b.get("name"))
            ))
        self.file.write("<dl>\n")
        for object in objects:
            self.writeobject(object, object.tag == "class")
        self.file.write("</dl>\n")
        # 2) class descriptions
        for object in objects:
            if object.tag != "class":
                continue
            name = object.get("name")
            localname = string.split(name, ".")[-1]
            anchor = name + "-class"
            self.file.write(
                "<h2><a id='%s' name='%s'>The %s Class</a></h2>\n" % (
                    anchor, anchor, localname
                    )
                )
            self.file.write("<dl>\n")
            self.writeobject(object)
            objects = []
            for object in object:
                info = object.find("info")
                if info is None or info.find("description") is None:
                    continue
                if object.tag not in ("method", "variable"):
                    continue
                objects.append(object)
            objects.sort(lambda a, b: cmp(
                string.lower(a.get("name")), string.lower(b.get("name"))
                ))
            for object in objects:
                if object.tag == "variable":
                    object.tag = "attribute"
                self.writeobject(object)
                if object.tag == "attribute":
                    object.tag = "variable"
            self.file.write("</dl>\n")

        if not zone:
            self.file.write("</body></html>\n")

        self.file.close()
        self.file = None

        return filename

##
# Prints a usage message and exits.

def usage():
    print "PythonDoc", VERSION, COPYRIGHT
    print
    print "Usage:"
    print
    print "  pythondoc [options] files..."
    print
    print "where the files can be either python modules or package"
    print "directories."
    print
    print "Options:"
    print
    print "  -p prefix    Prepend given prefix to symbol names."
    print "  -f           Generate output also for files without descriptions."
    print "  -x           Generate XML output (pythondoc infosets)."
    print
    print "  -s           Look for markup in docstrings (experimental)."
    print
    print "Output options:"
    print
    print "  -O format    Use given output format handler."
    print "  -D name      Define output variable."
    print "  -D name=text Set output variable to given text."
    print
    print "For more information on PythonDoc and the PythonDoc comment syntax,"
    print "see http://effbot.org/zone/pythondoc.htm"
    sys.exit(1)

if __name__ == "__main__":

    import getopt

    try:
        opts, args = getopt.getopt(sys.argv[1:], "D:fO:p:Vsx")
    except getopt.error:
        usage()

    force = 0
    prefix = None
    docstring = 0
    output_xml = 0
    output_handler = CompactHTML
    output_options = {}

    for k, v in opts:
        if k == "-f":
            force = 1
        elif k == "-p":
            prefix = v
        elif k == "-s":
            docstring = 1
        elif k == "-x":
            output_xml = 1
        elif k == "-O":
            try:
                m = __import__(v)
                for k in string.split(v, ".")[1:]:
                    m = getattr(m, k)
                output_handler = getattr(m, "PythonDocGenerator")
            except (ImportError, AttributeError):
                print "cannot find/load", repr(v), "generator"
                sys.exit(1)
        elif k == "-D":
            try:
                k, v = string.split(v, "=", 1)
            except ValueError:
                k = v; v = None
            output_options[k] = v
        elif k == "-V":
            DEBUG = DEBUG + 1

    if not args:
        usage()

    # instantiate output handler
    output_handler = output_handler(output_options)

    # check if handler supports custom tags
    try:
        TAGS = TAGS + output_handler.tags
    except AttributeError:
        pass

    import time
    t0 = time.time()

    input = output = 0

    for filename in args:

        this_prefix = prefix

        if os.path.isdir(filename):
            # FIXME: explicitly check if this is a package?
            files = glob.glob(os.path.join(filename, joinext("*", ".py")))
            if not this_prefix:
                this_prefix = os.path.basename(filename)
        else:
            if sys.platform == "win32" and glob.has_magic(filename):
                files = glob.glob(filename)
            else:
                files = [filename]

        files.sort()

        for file in files:

            try:
                module = parse(file, this_prefix, docstring=docstring)
            except IOError, v:
                sys.stderr.write("%s error: %s\n" % (file, v[1]))
                continue

            input = input + 1

            # check if any toplevel object has a description
            if not force:
                for n in module:
                    i = n.find("info")
                    if i and i.find("description") is not None:
                        break
                else:
                    continue # no documented subjects

            f = "pythondoc-" + string.replace(module.get("name"), ".", EXTSEP)

            if output_xml:
                # generate XML
                filename = joinext(f, ".xml")
                try:
                    out = open(filename, "w")
                    ElementTree.ElementTree(module).write(out)
                    out.close()
                except IOError, v:
                    sys.stderr.write("%s error: %s\n" % (filename, v[1]))
                else:
                    sys.stderr.write("%s ok\n" % filename)

            # generate output
            try:
                out = output_handler.save(module, f)
            except IOError, v:
                sys.stderr.write("%s error: %s\n" % (file, v[1]))
            else:
                if out:
                    sys.stderr.write("%s ok\n" % out)

            output = output + 1

    # flush output handler
    try:
        done = output_handler.done
    except AttributeError:
        pass
    else:
        out = output_handler.done()
        if out:
            sys.stderr.write("%s ok\n" % out)

    if DEBUG:
        sys.stderr.write(
            "%d files parsed, %d descriptions generated, in %.2f seconds\n" % (
                input, output, time.time() - t0
                ))



from sqlobject import SQLObject, StringCol, IntCol, connectionForURI

__connection__ = connectionForURI("sqlite:///:memory:")
hub = __connection__ 

class FirstClass(SQLObject):
    name = StringCol(title="This is a title",length=200,default="new name")

class SecondClass(SQLObject):
    name = StringCol()

class ThirdClass(SQLObject):
    name = StringCol()

FirstClass.createTable(ifNotExists=True)
SecondClass.createTable(ifNotExists=True)
ThirdClass.createTable(ifNotExists=True)



"Tests for SQLAlchemy support"

import cherrypy

from sqlalchemy import *
from sqlalchemy.ext.activemapper import ActiveMapper, column, one_to_many

from turbogears import config, redirect, expose, database, errorhandling
from turbogears.testutil import create_request, capture_log, print_log, \
                                sqlalchemy_cleanup
from turbogears.database import session, metadata, bind_meta_data
from turbogears.controllers import RootController

config.update({"sqlalchemy.dburi" : "sqlite:///:memory:"})

bind_meta_data()

metadata.engine.echo = True

users_table = Table("users", metadata,
    Column("user_id", Integer, primary_key=True),
    Column("user_name", String(40)),
    Column("password", String(10))
    )

class User(object):
    def __repr__(self):
        return "(User %s, password %s)" % (self.user_name, self.password)

usermapper = mapper(User, users_table)

class Person(ActiveMapper):
    class mapping:
        id = column(Integer, primary_key=True)
        name = column(String(40))
        addresses = one_to_many("Address")
    
class Address(ActiveMapper):
    class mapping:
        id = column(Integer, primary_key=True)
        address = column(String(40))
        city = column(String(40))
        person_id = column(Integer, foreign_key=ForeignKey("person.id"))

def setup_module():
    metadata.create_all()

def teardown_module():
    sqlalchemy_cleanup()

def test_query_in_session():
    i = users_table.insert()
    i.execute(user_name="globbo", password="thegreat!")
    query = session.query(User)
    globbo = query.select_by(user_name="globbo")[0]
    assert globbo.password == "thegreat!"
    users_table.delete().execute()

def test_create_and_query():
    i = users_table.insert()
    i.execute(user_name="globbo", password="thegreat!")
    s = users_table.select()
    r = s.execute()
    assert len(r.fetchall()) == 1
    users_table.delete().execute()

def test_active_mapper():
    p = Person(name="Ford Prefect")
    a = Address(address="1 West Guildford", city="Betelgeuse")
    p.addresses.append(a)
    session.flush()
    session.clear()
    q = session.query(Person)
    ford = q.select_by(name="Ford Prefect")[0]
    assert ford is not p
    assert len(ford.addresses) == 1

class MyRoot(RootController):
    def no_error(self, name):
        p = Person(name=name)
        raise redirect("/confirm")
    no_error = expose()(no_error)

    def e_handler(self, tg_exceptions=None):
        cherrypy.response.code = 501
        return "An exception ocurred: %r (%s)" % ((tg_exceptions,)*2)

    def create_person(self, **kw):
        id = kw.pop('id', None)
        if id is not None:
            kw['id'] = int(id)
        p = Person(**kw)
        return "No exceptions ocurred"
    create_person = expose()(create_person)
    create_person = errorhandling.exception_handler(e_handler)(create_person)

def test_implicit_trans_no_error():
    capture_log("turbogears.database")
    cherrypy.root = MyRoot()
    create_request("/no_error?name=A.%20Dent")
    print_log()
    session.clear()
    q = session.query(Person)
    arthur = q.select_by(name="A. Dent")[0]

def test_raise_sa_exception():
    capture_log("turbogears.database")
    cherrypy.root = MyRoot()
    create_request("/create_person?id=20")
    output = cherrypy.response.body[0]
    print output
    assert "No exceptions" in output

    create_request("/create_person?id=20")
    output = cherrypy.response.body[0]
    print output
    
    # Note that the specific DB2API may be either OperationalError or
    # IntegrityError depending on what version of sqlite and pysqlite
    # is used.
    assert "SQLError" in output
    assert cherrypy.response.code == 501

    



"Commands for listing TurboGears default and extension packages info"
import pkg_resources

entrypoints = {"tg-admin Commands" : "turbogears.command",
    "Template Engines" : "python.templating.engines", 
    "Widget Packages" : "turbogears.widgets", 
    "TurboGears Extensions" : "turbogears.extensions",
    "Identity Providers" : "turbogears.identity.provider",
    "Visit Managers" : "turbogears.visit.manager",
    "Toolbox Plugins" : "turbogears.toolboxcommand"}


def retrieve_info():
    """retrieve default and extension packages info"""
    
    # get default packages
    packages=['%s' % i for i in pkg_resources.require("Turbogears")]
    # get extension packages
    plugins = {}
    for name, pointname in entrypoints.items():
        plugins[name] = ["%s (%s)" % (entrypoint.name, str(entrypoint.dist))
            for entrypoint in pkg_resources.iter_entry_points(pointname)
        ]
    return packages, plugins

class InfoCommand:
    """Shows version info for debuging"""

    desc = "Show version info"

    def __init__(self,*args, **kwargs):
        pass

    def run(self):
        print """TurboGears Complete Version Information

TurboGears requires:
"""
        packages, plugins = retrieve_info()
        for p in packages:
            print '*', p
        for name, pluginlist in plugins.items():
            print "\n", name, "\n"
            for plugin in pluginlist:
                print '*', plugin



import os
import sys
import shutil
import time
from glob import glob
from setuptools import Command
import pkg_resources
pkg_resources.require("Kid >= 0.6.4")
import kid
import re
from distutils import log

class GenSite(Command):
    "setuptools command to generate the TurboGears website"
    
    user_options = [
        ("srcdirs=", "s", "directories containing the source files (default: docs)"),
        ("destdir=", "d", "destination output directory (default: dist/site)"),
        ("encoding=", "e", "encoding for output (default: utf8)"),
        ("force", "f", "regenerate all files"),
        ("ignoredirs=", "i", "directories to ignore (default: ['.svn', '.cvs'])"),
        ("ignorefiles=", "x", "files to ignore (default: ['.*\\.pyc', '.DS_Store'])"),
        ("nodelete=", "l", "directories to leave alone rather than delete"),
        ("templates=", "t", "mapping of templates to load (format: name=templatefile,name=templatefile)"),
        ("copydirs=", "c", "copy files from these directories without template proc. (destdir=srcdir,...)"),
        ("noprintable", "N", "don't make printable version of tutorials"),
        ("eggdir=", "g", "which directory has the eggs in it (default: '../thirdparty/eggs')")
    ]
    
    boolean_options=["force"]
    
    srcdirs = None
    destdir = "dist/site"
    encoding = "utf8"
    force = False
    ignoredirs = None
    ignorefiles = None
    nodelete = None
    templates = None
    copydirs = None
    eggdir = "../thirdparty/eggs"
    noprintable = False
    
    def initialize_options(self):
        pass
        
    def finalize_options(self):
        if self.srcdirs is None:
            self.srcdirs = ["docs"]
        if self.srcdirs == "":
            self.srcdirs = []
        self.ensure_string_list("srcdirs")
        self.ensure_string("destdir", "dist/site")
        self.ensure_string("encoding", "utf8")
        if self.ignoredirs is None:
            self.ignoredirs = [".svn", ".cvs"]
        self.ensure_string_list("ignoredirs")
        if self.ignorefiles is None:
            self.ignorefiles = ['.*\\.pyc', '.DS_Store']
        if self.nodelete is None:
            self.nodelete = ["dist/site/preview"]
        self.ensure_string_list("nodelete")
        self.ensure_string_list("ignorefiles")
        
        regexes = []
        for pat in self.ignorefiles:
            regexes.append(re.compile(pat))
        self.ignorepatterns = regexes
        
        self.templates, self.templates_order = self._split_mapping(self.templates, True)
        self.copydirs = self._split_mapping(self.copydirs)
            
    def _split_mapping(self, valToSplit, preserve_order=False):
        mapping = {}
        order = []
        if valToSplit and isinstance(valToSplit, basestring):
            pairs = re.split(",\s*", valToSplit)
            for pair in pairs:
                name, filename = re.split("\s*=\s*", pair)
                mapping[name] = os.path.abspath(filename)
                order.append(name)
        if preserve_order:
            return mapping, order
        return mapping
        
    def check_if_newer(self, src, dest):
        srcmtime = os.path.getmtime(src)
        if os.path.exists(dest):
            destmtime = os.path.getmtime(dest)
        else:
            destmtime = 0
        return srcmtime > destmtime
        
    def copy_if_newer(self, src, dest):
        if self.force or self.check_if_newer(src, dest):
            d = os.path.dirname(dest)
            if not os.path.exists(d):
                os.makedirs(d)
            self.copy_file(src, dest)
            
    def render_template(self, src, dest, depth):
        if not self.force and not self.check_if_newer(src, dest):
            return
        if not self.dry_run:
            log.info("rendering %s" % dest)
        else:
            log.info("skipping rendering %s" % dest)
            return
        
        template = kid.load_template(src, cache=False)
        template.Template.serializer = self.serializer
        toroot = "../" * depth
        destfile = dest[len(self.destdir)+1:]
        updated = time.strftime("%b %d, %Y", time.localtime(os.path.getmtime(src)))
        output = template.serialize(encoding=self.encoding, root=toroot, updated=updated,
            destfile=destfile, eggs=self.eggs)
        output = output.replace("$$", "$")
        destfile = open(dest, "w")
        destfile.write(output)
        destfile.close()
        
    def update_site_files(self, srcdir, processTemplates = True, destroot=None):
        if not destroot:
            destroot = self.destdir
        for root, dirs, files in os.walk(srcdir):
            if root != srcdir:
                fromroot = root[len(srcdir)+1:]
                segments = fromroot.split(os.sep)
                if set(segments).intersection(self.ignoredirs):
                    continue
                depth = len(segments)
            else:
                fromroot = ""
                depth = 0
            destdir = os.path.join(destroot, fromroot)
            if not os.path.exists(destdir):
                if not self.dry_run:
                    log.info("creating directory %s" % (destdir))
                    os.makedirs(destdir)
                else:
                    log.info("skipping creating directory %s" % (destdir))
                
            for file in files:
                ignore = False
                abs = os.path.abspath(file)
                for pat in self.ignorepatterns:
                    if pat.match(file):
                        ignore = True
                        break
                if ignore:
                    continue
                
                for tempfile in self.templates.values():
                    if tempfile == abs:
                        ignore = True
                        break
                if ignore:
                    continue
                
                ext = os.path.splitext(file)[1]
                dest = os.path.join(destdir, file)
                self.currentfiles.add(dest)
                if not processTemplates or ext != ".html":
                    self.copy_if_newer(os.path.join(root, file), 
                        dest)
                else:
                    self.render_template(os.path.join(root, file), 
                        dest, depth)

    def delete_excess_files(self):
        for root, dirs, files in os.walk(self.destdir):
            leavealone = False
            for dirname in self.nodelete:
                if root.startswith(dirname):
                    leavealone = True
                    break
            if leavealone:
                continue
            for file in files:
                dest = os.path.join(root, file)
                if dest not in self.currentfiles:
                    if not self.dry_run:
                        log.info("deleting %s" % dest)
                        os.unlink(dest)
                    else:
                        log.info("skipping deleting %s" % dest)

    def run(self):
        destdir = self.destdir
        log.info("generating website to %s" % destdir)

        if not os.path.exists(destdir):
            log.info("creating %s" % destdir)
            os.makedirs(destdir)
            
        for name in self.templates_order:
            filename = self.templates[name]
            log.info("template %s loaded as %s" % (filename, name))
            kid.load_template(filename, name=name)
        
        if self.eggdir:
            if not self.eggdir.endswith("/"):
                self.eggdir += "/"
            choplen = len(self.eggdir)
            self.eggs = [fn[choplen:] for fn in glob(self.eggdir + "*")]
            self.eggs.sort()

        self.currentfiles = set()

        self.serializer = kid.HTMLSerializer(encoding=self.encoding)

        for d in self.srcdirs:
            self.update_site_files(d)
        for dest, src in self.copydirs.items():
            if os.path.isdir(src):
                self.update_site_files(src, processTemplates=False, 
                                   destroot=os.path.join(self.destdir, dest))
            else:
                destfile = os.path.join(self.destdir, os.path.normpath(dest))
                self.copy_if_newer(src, destfile)
                self.currentfiles.add(destfile)
        self.printable_tutorial()
        self.delete_excess_files()
        
    def printable_tutorial(self):
        if self.noprintable:
            return
        self._make_printable(os.path.join("docs", "tutorials", "wiki20"), 3)
        self._make_printable(os.path.join("docs", "wiki20"))
    
    def _make_printable(self, tutdir, up_to_root=2):
        endpath = tutdir
        tutdir = os.path.join(self.srcdirs[0], tutdir)
        import cElementTree as elementtree
        masterdoc = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<?python import printable ?>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#" py:extends="printable">
<head>
 <meta content="text/html; charset=UTF-8" http-equiv="content-type" />
 <link rel="stylesheet" type="text/css" href="../../default.css" py:attrs="href=root+'default.css'"/>
 <link type="text/css" rel="stylesheet" href="../../sh/SyntaxHighlighter.css" py:attrs="href=root+'sh/SyntaxHighlighter.css'"></link> 
 <title>TurboGears: 20 Minute Wiki Tutorial</title>
</head>
<body>
"""
        docs = os.listdir(tutdir)
        docs.sort()
        for doc in docs:
            if not doc.endswith(".html"):
                continue
            log.info("combining %s" % doc)
            tree = elementtree.parse(os.path.join(tutdir, doc))
            body = tree.find("{http://www.w3.org/1999/xhtml}body")
            map(body.remove, body.findall("{http://www.w3.org/1999/xhtml}script"))
            bodytext = elementtree.tostring(body)
            bodytext = bodytext.replace("</html:body>", "")
            bodytext = bodytext.replace('<html:body xmlns:html="http://www.w3.org/1999/xhtml">', "")
            masterdoc += bodytext
            
        masterdoc += """<script src="../../sh/shCore.js" py:attrs="src=root+'sh/shCore.js'"></script>
<script src="../../sh/shBrushPython.js" py:attrs="src=root+'sh/shBrushPython.js'"></script>
<script src="../../sh/shBrushXml.js" py:attrs="src=root+'sh/shBrushXml.js'"></script>
<script src="../../sh/shBrushJScript.js" py:attrs="src=root+'sh/shBrushJScript.js'"></script>
<script language="javascript">
	dp.SyntaxHighlighter.HighlightAll('code');
</script>
</body></html>"""
        masterdoc = masterdoc.replace("html:", "")
        template = kid.Template(source=masterdoc, root="../" * up_to_root)
        template.serializer = self.serializer
        
        destend = os.path.join(self.destdir, endpath)
        if not os.path.exists(destend):
            os.makedirs(destend)
        outfn = os.path.join(destend, "printable.html")
        print "combined output: %s" % outfn
        outfile = open(outfn, "w")
        masterdoc = template.serialize(encoding=self.encoding)
        masterdoc = masterdoc.replace("$${", "${")
        outfile.write(masterdoc)
        outfile.close()
        self.currentfiles.add(outfn)
        


import cherrypy
import simplejson

from turbogears import controllers, expose
from turbogears.testutil import create_request, capture_log, get_log, \
                                print_log

class ExposeRoot(controllers.RootController):
    [expose("turbogears.tests.simple")]
    [expose("json")]
    def with_json(self):
        return dict(title="Foobar", mybool=False, someval="foo")

    [expose("turbogears.tests.simple")]
    [expose("json", accept_format = "text/javascript", as_format="json")]
    [expose('cheetah:turbogears.tests.textfmt', accept_format="text/plain")]
    def with_json_via_accept(self):
        return dict(title="Foobar", mybool=False, someval="foo")

cherrypy.root = ExposeRoot()
logged = None

def test_gettinghtml():
    global logged
    capture_log("turbogears.controllers")
    create_request("/with_json")
    logged = get_log()
    body = cherrypy.response.body[0]
    print body
    assert "Paging all foo" in body

def test_gettingjson():
    create_request("/with_json?tg_format=json")
    print "\n".join(logged)
    body = cherrypy.response.body[0]
    print body
    assert '"title": "Foobar"' in body

def test_gettingjsonviaaccept():
    create_request("/with_json_via_accept",
            headers=dict(Accept="text/javascript"))
    print "\n".join(logged)
    body = cherrypy.response.body[0]
    print body
    assert '"title": "Foobar"' in body

def test_getting_json_with_accept_but_using_tg_format():
    capture_log("turbogears.controllers")
    create_request("/with_json_via_accept?tg_format=json")
    print_log()
    print "\n".join(logged)
    body = cherrypy.response.body[0]
    print body
    assert '"title": "Foobar"' in body

def test_getting_plaintext():
    create_request("/with_json_via_accept",
        headers=dict(Accept="text/plain"))
    print "\n".join(logged)
    print cherrypy.response.body[0]
    assert cherrypy.response.body[0] == "This is a plain text for foo."
    
def test_allow_json():
    class NewRoot(controllers.RootController):
        [expose(template="turbogears.test.simple", allow_json=True)]
        def test(self):
            return dict(title="Foobar", mybool=False, someval="niggles")

    cherrypy.root = NewRoot()
    capture_log("turbogears.controllers")
    create_request("/test", headers= dict(accept="text/javascript"))
    print_log()
    print cherrypy.response.body[0]
    values = simplejson.loads(cherrypy.response.body[0])
    assert values == dict(title="Foobar", mybool=False, someval="niggles",
        tg_flash=None)
    assert cherrypy.response.headers["Content-Type"] == "text/javascript"

    create_request("/test?tg_format=json")
    print cherrypy.response.body[0]
    values = simplejson.loads(cherrypy.response.body[0])
    assert values == dict(title="Foobar", mybool=False, someval="niggles",
        tg_flash=None)
    assert cherrypy.response.headers["Content-Type"] == "text/javascript"




import threading
import copy
import warnings
import kid
from new import instancemethod
from itertools import ifilter, count
from turbogears import validators
from formencode.schema import Schema

try:
    set
except NameError:
    from sets import Set as set

__all__ = ["MetaWidget", "load_kid_template"]

param_prefix = '_param_'

class MetaWidget(type):
    def __new__(cls, name, bases, dct):
        # update_data has been deprecated
        if 'update_data' in dct and name != "Widget":
            warnings.warn(
                "update_data has been renamed update_params, please "
                "rename your method. "
                "Note: this warning will be removed once 1.0 is "
                "released and your actual code will stop working.",
                DeprecationWarning, 2)
        # Makes sure we get the union of params and member_widgets
        # from all our bases.
        params_set = set(dct.get('params', []))
        # template_vars has been deprecated
        if 'template_vars' in dct:
            params_set.update(dct['template_vars'])
            warnings.warn(
                "Use of template_vars inside a widget is deprecated, "
                "use params instead. "
                "Note: this warning will be removed once 1.0 is "
                "released and your actual code will stop working.",
                DeprecationWarning, 2)
        member_widgets_set = set(dct.get('member_widgets', []))
        compound = False
        for base in bases:
            params_set.update(getattr(base, 'params', []))
            if getattr(base, 'compound', False):
                member_widgets_set.update(getattr(base, 'member_widgets', []))
                compound = True
        for param in params_set:
            # Swap all params listed at 'params' with a ParamDescriptor
            try:
                dct[param_prefix+param] = dct[param]
                dct[param] = ParamDescriptor(param)
            except KeyError:
                # declared in a superclass, skip it...
                pass
        dct['params'] = list(params_set)
        #XXX: Remove when deprecation is effective
        dct['template_vars'] = dct['params']
        if compound:
            dct['member_widgets'] = list(member_widgets_set)
        # Pick params_doc from all bases giving priority to the widget's own
        params_doc = {}
        for base in bases:
            params_doc.update(getattr(base, 'params_doc', {}))
        params_doc.update(dct.get('params_doc', {}))
        dct['params_doc'] = params_doc
        return super(MetaWidget, cls).__new__(cls, name, bases, dct)

    def __init__(cls, name, bases, dct):
        if "__init__" in dct:
            dct["__init__"] = _decorate_widget_init(dct["__init__"])
            cls.__init__ = dct["__init__"]
        super(MetaWidget, cls).__init__(name, bases, dct)
        modname = "%s.%s" % (cls.__module__, name) 
        if cls.template:
            (cls.template_c,
             cls.template) = load_kid_template(cls.template, modname)
        cls._locked = False

#############################################################################
# Method decorators and other MetaWidget helpers                            #
#############################################################################

class ParamDescriptor(object):
    """Descriptor to support automatic callable support for widget params."""
    def __init__(self, param_name):
        self.param_name = param_prefix + param_name

    def __get__(self, obj, typ=None):
        if obj is None:
            # return the original class attribute. This makes the descriptor
            # almost invisible
            return getattr(typ, self.param_name)
        param = getattr(obj, self.param_name)
        if callable(param):
            return param()
        return param

    def __set__(self, obj, value):
        setattr(obj, self.param_name, value)

def lockwidget(self, *args, **kw):
    "Sets this widget as locked the first time it's displayed."
    gotlock = self._displaylock.acquire(False)
    if gotlock:
        del self.display
        self._locked = True
    output = self.__class__.display(self, *args, **kw)
    if gotlock:
        self._displaylock.release()
    return output

def _decorate_widget_init(func):
    """
    Ensures that the display method for the instance is overridden by
    lockwidget, that an eventual validator dict is applied to the
    widget validator and that a validation schema is generated for compound
    widgets.
    """
    def widget_init(self, *args, **kw):
        self.display = instancemethod(lockwidget, self, self.__class__)
        # We may want to move the InputWidget related logic to another
        # decorator
        input_widget = hasattr(self, "validator")
        # Makes sure we only generate a schema once in the all __init__ 
        # cooperative calls from subclasses. Same for the displaylock
        if not hasattr(self, '__initstack'):
            self._displaylock = threading.Lock()
            self.__initstack = []
        else:
            self.__initstack.append(True)
        # Manage an eventual validator dictionary that provides additional
        # parameters to the default validator (if present).
        # We remove it from kw and apply it after the execution of the
        # decorated __init__ method since only at this point we can check
        # for the presence of a default validator
        if input_widget and ("validator" in kw) \
           and isinstance(kw["validator"], dict):
            validator_dict = kw.pop("validator")
        else:
            validator_dict = None
        # Execute the decorated __init__ method
        func(self, *args, **kw)
        try:
            self.__initstack.pop()
        except IndexError:
            # We're the first __init__ called
            del self.__initstack
            if input_widget:
                # Apply an eventual validator dictionary
                if validator_dict:
                    if self.validator is not None:
                        class_validator = self.__class__.validator
                        if self.validator is class_validator:
                            # avoid modifying a validator that has been
                            # defined at class level and therefor is 
                            # shared by all instances of this widget
                            self.validator = copy.deepcopy(class_validator)
                        self.validator.__dict__.update(validator_dict)
                    else:
                        raise ValueError, ("You can't use a dictionary to "
                                           "provide additional parameters "
                                           "as the widget doesn't provide a "
                                           "default validator" )
                # Generate the validation Schema if we are a compound widget
                if getattr(self, 'compound', False):
                    widgets = self.iter_member_widgets()
                    # generate the schema associated to our widgets
                    validator = generate_schema(self.validator, widgets)
                    if getattr(self, 'repeating', False):
                        self.validator = validators.ForEach(validator)
                    else:
                        self.validator = validator
    return widget_init

#############################################################################
# Widget template support.                                                  #
#############################################################################

# Keeps a count of the widget instances which have overrided their template on
# the constructor so we can generate unique template-module names
widget_instance_serial = count()

# XXX: Maybe Widget should have a __del__ method to unload any template that
#      has been loaded during the instances initialization?
def load_kid_template(t, modname=None):
    """
    Loads the given template into the given module name, if modname is None,
    an unique one will be generated.
    Returns a tuple (compiled_tmpl, template_text (or modulepath)
    """
    if isinstance(t, basestring) and "<" in t:
        if not modname:
            modname = 'instance_template_%d' % widget_instance_serial.next()
        return (kid.load_template(t, name=modname).Template, t)
    else:
        return (t, None)

#############################################################################
# Schema generation support                                                 #
#############################################################################
class NullValidator(validators.FancyValidator):
    """ 
    A do-nothing validator. Used as a placeholder for fields with 
    no validator so they don't get stripped by the Schema.
    """
    if_missing = None

def copy_schema(schema):
    """ recursively copies a schema """
    new_schema = copy.copy(schema)
    new_schema.pre_validators = schema.pre_validators[:]
    new_schema.chained_validators = schema.chained_validators[:]
    fields = {}
    for k, v in schema.fields.iteritems():
        if isinstance(v, Schema):
            v = copy_schema(v)
        fields[k] = v
    new_schema.fields = fields
    return new_schema

def merge_schemas(to_schema, from_schema, inplace=False):
    """ 
    Recursively merges from_schema into to_schema taking care of leaving
    to_schema intact if inplace is False (default).
    """
    if not inplace: 
        to_schema = copy_schema(to_schema)

    # Recursively merge child schemas
    is_schema = lambda f: isinstance(f[1], validators.Schema)
    seen = set()
    for k, v in ifilter(is_schema, to_schema.fields.iteritems()):
        seen.add(k)
        from_field = from_schema.fields.get(k)
        if from_field:
            v = merge_schemas(v, from_field)
            to_schema.add_field(k, v)

    # Add remaining fields if we can
    can_add = lambda f: f[0] not in seen and can_add_field(to_schema, f[0])
    for field in ifilter(can_add, from_schema.fields.iteritems()):
        to_schema.add_field(*field)

    return to_schema

def add_field_to_schema(schema, widget):
    """ Adds a widget's validator if any to the given schema """
    name = widget.name
    if widget.validator is not None:
        if isinstance(widget.validator, validators.Schema): 
            # Schema instance, might need to merge 'em...
            if widget.name in schema.fields:
                assert isinstance(
                    schema.fields[name], validators.Schema
                ), "Validator for '%s' should be a Schema subclass" % name
                v = merge_schemas(schema.fields[name], widget.validator)
            else:
                v = widget.validator
            schema.add_field(name, v)
        elif can_add_field(schema, name):
            # Non-schema validator, add it if we can...
            schema.add_field(name, widget.validator)
    elif can_add_field(schema, name):
        schema.add_field(name, NullValidator())
            
def generate_schema(schema, widgets):
    """
    Generates or extends a copy of schema with all the validators from 
    the widget in the widgets list.
    """
    if schema is None:
        schema = validators.Schema()
    else:
        schema = copy_schema(schema)
    for widget in widgets:
        if widget.is_named:
            add_field_to_schema(schema, widget)
    return schema

def can_add_field(schema, field_name):
    """
    Checks if we can safely add a field. Makes sure we're not overriding
    any field in the Schema. NullValidators are ok to override. 
    """
    current_field = schema.fields.get(field_name)
    return bool(current_field is None or 
                isinstance(current_field, NullValidator))




import sys
import os
import re
import time
import codecs
import pygettext

MESSAGES = [] 

def detect_unicode_encoding(bytes):
    encodings_map = [
        (3, codecs.BOM_UTF8, 'UTF-8'),
        (4, codecs.BOM_UTF32_LE, 'UTF-32LE'),
        (4, codecs.BOM_UTF32_BE, 'UTF-32BE'),
        (2, codecs.BOM_UTF16_LE, 'UTF-16LE'),
        (2, codecs.BOM_UTF16_BE, 'UTF-16BE'),
    ]
    for (offset, bom, name) in encodings_map:
        if bytes[:offset] == bom:
            return name, offset
    return 'UTF-8', 0

class ParseError(ValueError):
    """Signals an error reading .po file."""

def merge(master_file,language_files):
    parsed_master_file =parse(master_file)
    for path in language_files:
        merging(parsed_master_file,path)

def merging(parsed_master_file,path):
    lang_file = parse(path)
    id_map = {}
    new_lang = []
    for msg in lang_file: id_map[msg['id']] = msg['message']
    for msg in parsed_master_file:
        msg['message'] = id_map.get(msg['id'])
        new_lang.append(msg)
    save(path,new_lang)

def items(path,sort_by,dir):
    po = parse(path)
    po = po[1:]
    if sort_by: return sort(po,sort_by,dir)
    return po 

def sort(po,sort_by,dir):
    group = {}
    sorted =[]
    col_map  = {
                'id':'id',
                'string':'message',
                'context':'path'
               }
    for message in po:
        group.setdefault(message[col_map[sort_by]],[]).append(message)

    kg = group.keys()
    kg.sort()
    if dir=='up': kg.reverse()
    for k in kg: sorted.extend(group[k])
    return sorted

def save(path,message_list):
    txt = []
    m = message_list[0]['message']
    txt.append(m)
    txt.append(u'\n\n');

    for p  in message_list[1:]:
        message = p['message'] or ''
        context = p['context']
        id = p['id']
        txt.append(u'#: %s'% context) 
        txt.append(u'msgid %s\n'% normalize(id) )
        txt.append(u'msgstr %s\n\n'% normalize(message) )
    txt=u''.join(txt)
    
    backup_name = path.replace('.po','.back')
    try:
        os.remove(backup_name)
    except os.error:
        pass
    os.rename(path,backup_name)
    codecs.open(path,'wb', 'utf-8').write(txt)

def update(path,msg_id,msg_text):
    message_list = parse(path)
    for p in message_list[1:]:
        if p['id'].strip() == msg_id.strip(): p['message'] = msg_text 
    save(path,message_list)

def quote(msg):
    return pygettext.escape_unicode(msg)

def normalize(s):
    # taken from pygettext module but changed a bit
    lines = s.split('\n')
    if len(lines) == 1:
        s = '"' + quote(s) + '"'
    else:
        if not lines[-1]:
            del lines[-1]
            lines[-1] = lines[-1] + '\n'
        for i in range(len(lines)):
            lines[i] = quote(lines[i])
        lineterm = '\\n"\n"'
        s = '""\n"' + lineterm.join(lines) + '"'
    return s

def add(id, str, context, fuzzy,MESSAGES):
    "Add a non-fuzzy translation to the dictionary."
    if fuzzy: return
    c = context.split(':')
    path =c[0]
    file = os.path.basename(path) 
    line=c[-1].replace('\n','') #remove the \n
    
    MESSAGES.append( {'id':id,
                    'message':str,
                    'path':path,
                    'context':context,
                    'file':file,
                    'line':line}
                   )

def parse(infile):
    MESSAGES = []
    ID = 1
    STR = 2
    header = []
    
    fd = open(infile, 'rt')
    encoding, offset = detect_unicode_encoding(fd.read(4))
    fd.seek(offset)
    lines = [line.decode(encoding) for line in fd.readlines()]

    section = None
    fuzzy = 0

    # Parse the catalog
    lno = 0
    context = ''
    prev_context = ''
    heading = True
    for l in lines:
        if not l: continue
        lno += 1
        if heading:
            if l.startswith('#: '): heading=False
            if l.startswith('msgid "') and header and 'Generated-By:' in header[-1]:heading=False
            if l.strip() and heading: header.append(l)
            
        # If we get a comment line after a msgstr, this is a new entry
        if l[0] == '#' and section == STR:
            add(msgid, msgstr,prev_context, fuzzy,MESSAGES)
            section = None
            fuzzy = 0

        # Record a fuzzy mark
        if l[:2] == '#,' and l.find('fuzzy'):
            fuzzy = 1
        if l.startswith('#: '): context = l[len('#: '):]
        # Skip comments
        if l[0] == '#':
            continue
        # Now we are in a msgid section, output previous section
        if l.startswith('msgid'):
            if section == STR:
                add(msgid, msgstr, prev_context, fuzzy,MESSAGES)
            section = ID
            prev_context = context 
            l = l[5:]
            msgid = msgstr = ''
        # Now we are in a msgstr section
        elif l.startswith('msgstr'):
            section = STR
            l = l[6:]
        # Skip empty lines
        l = l.strip()
        if not l:
            continue
        # XXX: Does this always follow Python escape semantics?
        try:
            l = eval(l)
        except Exception, e:
            print >> sys.stderr, 'Escape error on %s:%d' % (infile, lno), \
                  'before:', `l`
            raise ParseError(e)
        try:
            l = l.decode('utf8')
        except UnicodeDecodeError:
            print >> sys.stderr, 'Encoding error on %s:%d' % (infile, lno), \
                  'before:', `l`
            raise ParseError(e)
        if section == ID:
            msgid += l
        elif section == STR:
            msgstr += l
        else:
            print >> sys.stderr, 'Syntax error on %s:%d' % (infile, lno), \
                  'before:'
            print >> sys.stderr, l
            raise ParseError(e)
    # Add last entry
    if section == STR:
        add(msgid, msgstr, prev_context, fuzzy,MESSAGES)
    
    MESSAGES[0]['message'] = u''.join(header)
    return MESSAGES




# $Id$
# a simple python source code colorizer

import sys, tokenize, keyword

# TODO: add support for proper rendering of pythondoc comments
# TODO: add support for source code encodings

# this code works under Python 2.2 and later
try:
    enumerate
except NameError:
    def enumerate(seq):
        return [(i, seq[i]) for i in range(len(seq))]

##
# Colorizer base class.

class Colorizer:

    def colorize(self):

        pos = (1, 0)
        cls = None
        tok = None

        source = tokenize.generate_tokens(self.readline)

        self.begin()

        self.lineno(1)
        for type, token, start, end, line in source:
            # FIXME: check source code encoding

            # print >>sys.stderr, tokenize.tok_name[type], repr(token)

            # note: since we're skipping empty tokens, empty lines at
            # the end of the file will be ignored.  to avoid this,
            # change the following test to
            #
            # if type != tokenize.ENDMARKER and (not token or token == "\n"):

            if not token or token == "\n":
                continue

            # assert token != tokenize.NL and token !=tokenize.NEWLINE
            if start != pos:
                self.style(None, None)
                # insert missing whitespace
                if start[0] == pos[0]:
                    self.write(" "*(start[1]-pos[1]))
                else:
                    for i in range(pos[0], start[0]):
                        self.write("\n")
                        self.lineno(i+1)
                    self.write(" "*start[1])

            # figure out what style to use, if any
            if type == tokenize.STRING:
                self.style("string", None)
            elif type == tokenize.NAME:
                if keyword.iskeyword(token):
                    self.style("keyword", token)
                elif tok == "class":
                    self.style("class", token)
                elif tok == "def":
                    self.style("function", token)
                elif tok == "@":
                    self.style("decorator", token)
            elif type == tokenize.COMMENT:
                self.style("comment", None)
            else:
                self.style(None, None)

            # process token parts
            parts = token.split("\n")
            for ix, part in enumerate(parts):
                if not ix:
                    self.write(part)
                elif part or ix < len(parts)-1:
                    cls = self.style(None, None)
                    self.write("\n")
                    self.lineno(start[0]+ix)
                    self.style(cls, None)
                    self.write(part)
            pos = end
            tok = token

        self.style(None, None) # reset style
        self.end()

    ##
    # (hook) Called to render a line number.

    def lineno(self, lineno):
        pass # enter a new line

    ##
    # (hook) Called to read a line from the source.

    def readline(self):
        return sys.stdin.readline()

    ##
    # (hook) Called to write a line of text to the target.

    def write(self, text):
        sys.stdout.write(text)

    ##
    # (hook) Called to change to a new style.
    #
    # @param style The requested style.  The value None means "plain
    #   text".  Other style values are "class", "comment", "function",
    #   "decorator", "keyword", and "string".
    # @param token The actual token that caused the colorizer to
    #   request this style.  This is only set for "class", "function",
    #   "decorator", and "keyword" styles; for other styles, this
    #   parameter is None.
    # @return The old style (before this call was made).

    def style(self, style, token):
        pass

    ##
    # (hook) Called before colorization starts.

    def begin(self):
        pass

    ##
    # (hook) Called after colorization ends.  The style is set to None
    # before this method is called.

    def end(self):
        pass

##
# A colorizer that reads from, and writes to file streams.

class FileColorizer(Colorizer):

    def __init__(self, infile, outfile):

        if not hasattr(infile, "read"):
            infile = open(infile)

        def readline():
            s = infile.readline()
            return s and s.expandtabs().rstrip() + "\n"
        self.readline = readline

        if not hasattr(outfile, "write"):
            outfile = open(outfile, "w")
        self.write = outfile.write

##
# A colorizer that reads from a file (or a stream), and renders output
# as HTML.

class HtmlColorizer(FileColorizer):

    cls = None

    def __init__(self, infile, outfile):

        FileColorizer.__init__(self, infile, outfile)

        # replace write with an escaping
        import cgi
        self.write_raw = self.write
        def write(text):
            # FIXME: use encode(xmlcharrefreplace) as well ?
            self.write_raw(cgi.escape(text))
        self.write = write

    def style(self, cls, token):
        current = self.cls
        if cls == self.cls:
            return current
        if self.cls:
            self.write_raw("</span>")
        if cls:
            self.write_raw("<span class='%s'>" % cls)
        self.cls = cls
        return current

    def begin(self):
        self.write_raw("<pre class='python'><code>")

    def end(self):
        self.write_raw("</code></pre>")

    def lineno(self, line):
        self.write_raw(
            "<span id='line%s' class='lineno'>%05d</span> " % (line, line)
            )

if __name__ == "__main__":
    # sanity check
    PYTHONWORKS_STYLE = """
    .class {color:firebrick;font-weight:bold;}
    .comment {color:darkolivegreen;font-style:italic;}
    .decorator {color:firebrick;}
    .function {color:firebrick;font-weight:bold;}
    .keyword {color:firebrick;}
    .lineno {color:gray;border-right:1px solid gray;padding-right:5px;}
    .string {color:navy;}
    """
    TRAC_STYLE = """
    .class {color:#900;font-weight:bold;border-bottom:none;}
    .comment {color:#998;font-style:italic;}
    .decorator {color:#900;border-bottom:none;}
    .function {color:#900;font-weight:bold;border-bottom:none;}
    .keyword {color:#000;font-weight:bold;}
    .lineno {color:black;}
    .string {color:#b84;font-weight:bold;}
    }
    """
    try:
        file = sys.argv[1]
    except IndexError:
        file = "pythondoc.py"
    print "<html><head>"
    print "<style>", PYTHONWORKS_STYLE, "</style>"
    print "</head><body>"
    c = HtmlColorizer(file, sys.stdout)
    import time
    t0 = time.clock()
    c.colorize()
    t0 = time.clock() - t0
    print "</body></html>"
    print >>sys.stderr, t0



try:
    from sqlobject import *
    from datetime import *

    from turbogears.database import PackageHub

    # create a PackageHub
    hub = PackageHub("turbogears.i18n.sogettext")
    __connection__ = hub

    class TG_Domain(SQLObject):

        name = StringCol(alternateID=True)
        messages = MultipleJoin("TG_Message")

        class sqlmeta:
            table="tg_i18n_domain"
            defaultOrder="name"

    class TG_Message(SQLObject):

        name = UnicodeCol()
        text = UnicodeCol(default="")
        domain = ForeignKey("TG_Domain")
        locale = StringCol(length=15)
        created= DateTimeCol(default=datetime.now)
        updated=DateTimeCol(default=None)

        def _set_text(self, text):

            self._SO_set_text(text)
            self.updated = datetime.now()

        class sqlmeta:
            table="tg_i18n_message"
            defaultOrder="name"

except ImportError:
    pass




from turbogears.i18n.tg_gettext import *
from turbogears.i18n.tests import setup_module

def test_gettext():
    assert unicode(gettext("Welcome")) == "Welcome"
    assert unicode(gettext("Welcome", "en")) == "Welcome"
    assert unicode(gettext("Welcome", "fi")) == "Tervetuloa"
    assert unicode(gettext("Welcome", "fi_FI")) == "Tervetuloa"

def test_invalid_domain():
    assert gettext("Welcome", "fi", "fubar") != "Tervetuloa"

def test_is_unsupported_locale():
    assert is_locale_supported("en") == True
    assert is_locale_supported("en-gb") == False
    assert is_locale_supported("de") == False
    assert is_locale_supported("en", "fubar") == False

def test_gettext_unsupported_locale():
    assert unicode(gettext("Welcome", "en-gb")) == "Welcome"
    assert unicode(gettext("Welcome", "de")) == "Welcome"

def test_ngettext(): 
    assert ngettext("You have %i apple", \
            "You have %i apples", 1) %1 == "You have 1 apple"
    assert ngettext("You have %i apple", \
            "You have %i apples", 1, "fi") %1 == "Sinulla on 1 omena"
    assert ngettext("You have %i apple", \
            "You have %i apples", 3) %3 == "You have 3 apples"
    assert ngettext("You have %i apple", \
            "You have %i apples", 3, "fi") %3== "Sinulla on 3 omenaa"

def test_lazystring():
    s1 = lazystring("simple".upper)
    assert s1 == 'SIMPLE'
    assert s1 != 'HARD'
    assert s1 > 'HARD' or s1 < 'HARD'
    # should lazystring support string concatenation?
    # assert s1 + '!' == 'SIMPLE!'
    # assert 'TOO ' + s1 == 'TOO SIMPLE'



import turbogears
from turbogears import controllers
import sys
import StringIO
from code import InteractiveConsole
from textwrap import TextWrapper

class WebConsole(controllers.RootController):
    """Web based Python interpreter"""
    __label__='WebConsole'
    
    icon = "/tg_static/images/shell.png"

    def __init__(self, width=80):
        self.console = None

        try:
            sys.ps1
        except AttributeError:
            sys.ps1 = '>>> '
        try:
            sys.ps2
        except AttributeError:
            sys.ps2 = '... '
        
    def process_request(self, line):
        more, output = self._process_request(line)
        return dict(more=more, output=output)
    process_request = turbogears.expose()(process_request)

    def process_multiline_request(self, block):
        outlines = []
        response = ''
        
        lines = [line for line in block.split('\n')]
        
        for line in lines:	
            more, output = self._process_request(line)
            
            if output[-1] == '\n':     # we'll handle the newlines later.
                output = output[:-1]
            
            outlines.append(output)
        
        return dict(more=more,output='\n'.join(outlines))
    process_multiline_request = turbogears.expose()(process_multiline_request)

    def _process_request(self, line):
        if len(self.console.buffer):
            prompt = sys.ps2
        else:
            prompt = sys.ps1
            
        myout = StringIO.StringIO()
        
        output = "%s%s" % ( prompt, line )
        # hopefully python doesn't interrupt in this block lest we'll get some curious output.
        try:
            sys.stdout = myout
            sys.stderr = myout
            more = self.console.push(line)
        finally:
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__
        
        stdout = myout.getvalue()
        
        if stdout: output = '%s\n%s' % ( output, stdout )
        
        return ( more, output.rstrip() )

    def new_console(self):
        locs = dict(__name__='tg-admin',__doc__=None,reload_console=self.new_console)
        mod = turbogears.util.get_model()
        if mod:
            locs.update(mod.__dict__)
        self.console = InteractiveConsole(locals=locs)

    def index(self):
        if not self.console:
            self.new_console()
        return dict()
    index = turbogears.expose(html="turbogears.toolbox.console")(index)



import sys
import os
import optparse

from apigen.pythondoc import ModuleParser, CompactHTML, ElementTree
from apigen.element_colorizer import ElementColorizer

from apigen import release

ET = ElementTree

def myparse(file, output):

    basename = os.path.basename(file)
    basename = os.path.splitext(basename)[0]
    sourcefile = "%s-source.html" % basename
    parser = ModuleParser(file)
    module = parser.parse(docstring=1)

    for elem in module.getiterator():

        lineno = elem.get("lineno")
        if not lineno:
            continue

        description = elem.find("info/description")
        if description is None:
            description = ET.SubElement(elem, "p")
        if len(description) == 0 and description.text:
            # wrap it in a paragraph
            p = ET.SubElement(description, "p")
            p.text = description.text
            description.text = None

        # insert hyperlink
        href = sourcefile + "#" + lineno
        p = ET.SubElement(description, "p")
        a = ET.SubElement(p, "a", href=href)
        a.set("class", "sourcelink")
        a.text = "View source code..."

    # ET.ElementTree(module).write("out.xml")

    formatter = CompactHTML()
    print formatter.save(module, os.path.join(output, basename)), "ok"
    c = ElementColorizer(open(file).read())
    pre = c.colorize()
    sourceoutput = os.path.join(output, sourcefile)
    root = ET.Element("html")
    head = ET.SubElement(root, "head")
    style = ET.SubElement(head, "style")
    style.attrib["type"] = "text/css"
    style.text = """
    .class {color:firebrick;font-weight:bold;}
    .comment {color:darkolivegreen;font-style:italic;}
    .decorator {color:firebrick;}
    .function {color:firebrick;font-weight:bold;}
    .keyword {color:firebrick;}
    .lineno {color:gray;border-right:1px solid gray;padding-right:5px;}
    .string {color:navy;}
    """
    body = ET.SubElement(root, "body")
    body.append(pre)
    tree = ET.ElementTree(root)
    tree.write(sourceoutput)
    print sourceoutput, "ok"

def generate_index(package, packages, files, output):
    html = ET.Element("html")
    tree = ET.ElementTree(html)
    body = ET.SubElement(html, "body")
    h1 = ET.SubElement(body, "h1")
    h1.text = package
    if packages:
        h2 = ET.SubElement(body, "h2")
        h2.text = "Subpackages"
        dl = ET.SubElement(body, "dl")
        
        for p in packages:
            dt = ET.SubElement(dl, "dt")
            a = ET.SubElement(dt, "a")
            a.attrib["href"] = "%s/index.html" % (p)
            a.text = p
            dd = ET.SubElement(dl, "dd")
            # add package description here... will require cooperation
            # and aggregation
    files.sort()
    if files:
        h2 = ET.SubElement(body, "h2")
        h2.text = "Modules"
        dl = ET.SubElement(body, "dl")
        
        for f in files:
            f = os.path.splitext(f)[0]
            dt = ET.SubElement(dl, "dt")
            a = ET.SubElement(dt, "a")
            a.attrib["href"] = "%s.html" % (f)
            a.text = f
            dd = ET.SubElement(dl, "dd")
            # add module description here... will require cooperation
            # and aggregation
    indexfile = os.path.join(output, "index.html")
    tree.write(indexfile)
    print indexfile, "ok"

def process_files(roots, output, ignore):
    splitext = os.path.splitext
    for thisroot in roots:
        for root, dirs, files in os.walk(thisroot):
            if root in ignore:
                continue
            if ".svn" in dirs:
                dirs.remove(".svn")
            files = filter(lambda file: splitext(file)[1] == ".py", files)
            if "__init__.py" in files:
                packages = []
                for dir in dirs:
                    subdir = os.path.join(root, dir)
                    if os.path.exists(os.path.join(subdir,
                            "__init__.py")):
                        if not subdir in ignore:
                            packages.append(dir)
                initiallen = len(os.path.commonprefix([thisroot, root]))
                remainder = root[initiallen:]
                if remainder.startswith("/"):
                    remainder = remainder[1:]
                currentoutput = os.path.join(output, remainder)
                if not os.path.exists(currentoutput):
                    os.makedirs(currentoutput)
                package = os.path.basename(root)
                generate_index(package, packages, files, currentoutput)
            for file in files:
                myparse(os.path.join(root, file), currentoutput)

def main():
    parser = optparse.OptionParser(
        usage="%prog [-o]", version="%prog " + release.version)
    parser.add_option("-o", "--output",
             help="output directory (will be created)",
             dest="output")
    parser.add_option("-i", "--ignore",
        help="ignore this path/file", dest="ignore", action="append")
    (options, args) = parser.parse_args()
    process_files(args, options.output, options.ignore)


from turbogears import validators

def test_booleanstrings():
    """'False', 'false', 'True', 'true' should be proper boolean values"""
    b = validators.StringBoolean(if_empty=False)
    assert 'false' == b.from_python(False)
    assert 'true' == b.from_python(True)
    assert 'false' == b.from_python(0)
    assert 'true' == b.from_python(1)
    assert b.to_python("True") is True
    assert b.to_python("False") is False
    assert b.to_python("true") is True
    assert b.to_python("false") is False
    assert b.to_python("") is False
    assert b.to_python(None) is False
    
    try:
        b.to_python("foobar")
        assert False, "random strings should fail validation"
    except validators.Invalid:
        pass

def test_datetimeconverter():
    import datetime
    date = datetime.datetime(2005,11,22,18,22)
    dt  = validators.DateTimeConverter()
    assert date == dt.to_python(date), "Accepts datetime OK"
    assert date == dt.to_python(dt.from_python(date)), "Good datetime passes validation"

    try:
        dt.to_python("foo")
        assert False, "random strings should fail validation"
    except validators.Invalid:
        pass

def test_jsonvalidator():
    v = validators.JSONValidator()
    origlist = ["Foo", "Bar", "Baz"]
    json = v.from_python(origlist)
    print json
    assert json == '["Foo", "Bar", "Baz"]'
    l = v.to_python(json)
    print l
    assert origlist == l

def test_unicodestring_validator():
    v = validators.UnicodeString()
    assert u'TurboGears' == v.to_python('TurboGears')
    assert 'TurboGears' == v.from_python('TurboGears')
    v = validators.UnicodeString(inputEncoding='cp1251')
    assert repr(v.to_python('\xf0\xf3\xeb\xe8\xf2')) == \
            "u'\u0440\u0443\u043b\u0438\u0442'"
    assert v.from_python(u'\u0440\u0443\u043b\u0438\u0442') == \
        '\xd1\x80\xd1\x83\xd0\xbb\xd0\xb8\xd1\x82' # in utf-8 encoding
    v = validators.UnicodeString()
    try:
        # feed cp1251 encoded data when utf8 is expected
        print `v.to_python('\xf0\xf3\xeb\xe8\xf2')`
        assert 0, 'malformed data not detected'
    except validators.Invalid:
        pass

def test_number_validador():
    assert validators.Number.to_python("45") == 45
    # test for ticket #955
    assert validators.Number.to_python(45) == 45




import kid
from turbogears.i18n.kidutils import *
from turbogears.i18n.tests import setup_module

def test_match_template():

    template = """
    <?python
    from turbogears.i18n import translate
    ?>
    <html xmlns:py="http://purl.org/kid/ns#">
        <translate py:match="item.attrib.has_key('lang')" py:replace="translate(item, 'lang')"/>
        <body lang="">
            Welcome
            <p>Welcome</p>  
            <p lang="en">Welcome</p>
            <p lang="fi">Welcome</p>
        </body>
    </html>
    """

    t = kid.Template(source = template)
    output = t.serialize()
    print output
    assert '<p lang="en">Welcome</p>' in output
    assert '<p lang="fi">Tervetuloa</p>' in output

def test_i18n_filter():

    template = """
    <html xmlns:py="http://purl.org/kid/ns#">
        <body>
            Welcome, <em>guest</em>!
            <p>Welcome</p>  
            <p lang="en">Welcome</p>
            <p lang="fi">Welcome<a href="" lang="de">Welcome</a></p>
        </body>
    </html>"""

    t = kid.Template(source = template)
    t._filters+=[i18n_filter]
    output = t.serialize()
    print output




# -*- coding: utf-8 -*- 

from turbogears import view, config
import unittest

class TestView(unittest.TestCase):

    def test_UnicodeValueAppearingInATemplateIsFine(self):
        ustr = u"micro-eXtreme Programming ( µ XP): Embedding XP Within Standard Projects"
        info = dict(someval=ustr)
        val = view.render(info, template="turbogears.tests.simple")
        self.failUnless(u"Paging all " + ustr in val.decode("utf-8"))

    def test_templateRetrievalByPath(self):
        config.update({'server.environment' : 'development'})
        from turbokid import kidsupport
        ks = kidsupport.KidSupport()
        cls = ks.load_template("turbogears.tests.simple")
        assert cls is not None
        t = cls()
        t.someval = "hello"
        filled = str(t)
        assert "groovy" in filled
        assert "html" in filled
        
        # the following import should not fail, if everything is working correctly.
        import turbogears.tests.simple

    def test_default_output_encoding(self):
        info = dict(someval="someval")
        # default encoding is utf-8
        val = view.render(info, template="turbogears.tests.simple")
        assert 'utf-8' in view.cherrypy.response.headers["Content-Type"]
        config.update({'tg.defaultview':'kid', 'kid.encoding':'iso-8859-1'})
        val = view.render(info, template="turbogears.tests.simple")
        assert 'iso-8859-1' in view.cherrypy.response.headers["Content-Type"]



"""
Localized formatting functions. These functions extract localization data from config files 
located in the data/directory.  
"""

from turbogears.i18n.utils import get_locale
import pkg_resources
import os
import re

def is_locale_format(locale):

    py_filename = pkg_resources.resource_filename("turbogears.i18n.data", "%s.py" %locale)
    if os.path.exists(py_filename):return True
    
    pyc_filename = pkg_resources.resource_filename("turbogears.i18n.data", "%s.pyc" %locale)
    if os.path.exists(pyc_filename):return True
    
    return False
    
def get_locale_module(locale):

    try:
        # check if locale is supported. If not, check again with first part of locale
        # for example, "fi_FI" > "fi".
        
        if not is_locale_format(locale):
        
           locale = locale[:2]
           
        name = "turbogears.i18n.data.%s" %locale
        
        mod = __import__(name)
        parts = name.split(".")[1:]
        for p in parts:mod = getattr(mod, p)
        return mod
    
    except (ImportError, AttributeError):
        return None
        
def get(locale, name, default=None):

    locale = get_locale(locale)
    mod = get_locale_module(locale)
    
    return getattr(mod, name, default)
    
def get_countries(locale=None):
    """
    Returns list of tuples, consisting of international country code and localized 
    name, e.g. ('AU', 'Australia')
    """
    
    countries = get(locale, "countries", {}).items()
    countries.sort(lambda x,y:cmp(x[1], y[1]))
    return countries  
    
def get_country(key, locale=None):
    """
    Returns localized name of country based on international country code
    """
    
    return get(locale, "countries", {})[key]
    
def get_languages(locale=None):
    """Returns list of tuples, with language code and localized name, e.g. ('en', 'English')
    """
    
    languages = get(locale, "languages", {}).items()
    languages.sort(lambda x,y:cmp(x[1], y[1]))
    return languages  

def get_language(key, locale=None):
    """
    Returns localized name of language based on language code
    """
    
    return get(locale, "languages", {})[key]
        
def get_month_names(locale=None):
    """Returns list of full month names, starting with January
    """
    return get(locale, "months", [])
    
def get_abbr_month_names(locale=None):
    """Returns list of abbreviated month names, starting with Jan
    """
    
    return get(locale, "abbrMonths", [])
    
def get_weekday_names(locale=None):
    """Returns list of full weekday names
    """
            
    return get(locale, "days", [])
    
def get_abbr_weekday_names(locale=None):
    """Returns list of abbreviated weekday names
    """

    return get(locale, "abbrDays", get_weekday_names(locale))
    
def get_decimal_format(locale=None):

    return get(locale, "numericSymbols").get("decimal", ".")

def get_group_format(locale=None):

    return get(locale, "numericSymbols").get("group", ",")


def format_number(value, locale=None):
    
    """
    Returns number formatted with grouping for thousands, e.g. 5000000>5,000,000
    """
   
    gf = get_group_format(locale)

    thou=re.compile(r"([0-9])([0-9][0-9][0-9]([%s]|$))" %gf).search

    v=str(value)
    mo=thou(v)
    while mo is not None:
        l = mo.start(0)
        v=v[:l+1]+gf+v[l+1:]
        mo=thou(v)
    return unicode(v)


def format_decimal(value, num_places, locale=None):

    """
    Returns number formatted with grouping for thousands and correct 
    notation, e.g. 5000000.898>5,000,000.898
    """

    format = "%%.%df"%num_places
    str = format%value        
    num, decimals = str.split(".")
    return unicode(format_number(num, locale) + get_decimal_format(locale) + decimals)
    
def format_currency(value, locale=None):

    """
    Returns formatted currency value
    """
    
    return format_decimal(value, 2, locale)

def parse_number(value, locale=None):

    """
    Takes localized number string and returns a long integer  (or throws ValueError if bad format)
    """
    
    return long(value.replace(get_group_format(locale), ""))
    
def parse_decimal(value, locale=None):

    """
    Takes localized decimal string and returns a float  (or throws ValueError if bad format)
    """

    value = value.replace(get_group_format(locale), "")
    value = value.replace(get_decimal_format(locale), ".")
    return float(value)
    
def get_date_format(format, locale=None):

    formats = get(locale, "dateFormats", {})
    return formats.get(format, None)

def format_date(dt, format="medium", locale=None, time_format="", date_format=""):
    """Returns formatted date value.

    Format can be "full", "long", "medium" or "short".  To have complete control over
    formatting, use time_format and date_format parameters.

    @param dt: datetime
    @type dt: datetime.datetime
    @param format: format("full", "long", "medium", "short")
    @type format:string
    @param locale
    @param time_format: standard time formatting string, e.g. %H:%M
    @type time_format:string
    @param time_format: date formatting template string. Template variables include
    standard date formatting string like %d or %Y plus a few locale-specific names:
    %%(abbrmonthname)s, %%(dayname)s, %%(abbrmonthname)s and %%(monthname)s.
    @type time_format:string
    """

    if date_format:
        pattern = date_format
    else:
        pattern = get_date_format(format, locale)

    if not pattern:return str(dt)

    month = dt.month-1
    weekday = dt.weekday()

    # becase strftime() accepts str only but not unicode
    # we encode string to utf-8 and then decode back
    date_str = dt.strftime(pattern.encode('utf8')+time_format)
    return date_str.decode('utf8') % {
        'monthname':get_month_names(locale)[month],
        'abbrmonthname':get_abbr_month_names(locale)[month],
        'dayname':get_weekday_names(locale)[weekday],
        'abbrdayname':get_abbr_weekday_names(locale)[weekday],
    }

